@@ -69,6 +69,14 @@ class TestCase_launch(unittest.TestCase):
         else:
             self.fail("svnmerge.launch did not cause a LaunchError as expected")
 
+class TestCase_PrefixLines(unittest.TestCase):
+    def test_basic(self):
+        self.assertEqual("zz\n", svnmerge.prefix_lines("zz", "\n"))
+        self.assertEqual("zzfoo\n", svnmerge.prefix_lines("zz", "foo\n"))
+        self.assertEqual("zzfoo\nzzbar\n", svnmerge.prefix_lines("zz", "foo\nbar\n"))
+        self.assertEqual("zz\nzzfoo\n", svnmerge.prefix_lines("zz", "\nfoo\n"))
+        self.assertEqual("zz\nzzfoo\nzzbar\n", svnmerge.prefix_lines("zz", "\nfoo\nbar\n"))
+
 class TestCase_RevisionSet(unittest.TestCase):
     def test_constr_string(self):
         rs = svnmerge.RevisionSet("10- 15, 12-48,2 ")
@@ -76,6 +84,11 @@ class TestCase_RevisionSet(unittest.TestCase):
         self.assert_(2 in rs)
         self.assert_(9 not in rs)
 
+        rs = svnmerge.RevisionSet("10: 15, 12:48,2 ")
+        self.assert_(17 in rs)
+        self.assert_(2 in rs)
+        self.assert_(9 not in rs)
+
     def test_constr_dict(self):
         rs = svnmerge.RevisionSet({18:1, 24:1, 25:1, 43:1})
         self.assert_(24 in rs)
@@ -87,11 +100,29 @@ class TestCase_RevisionSet(unittest.TestCase):
         self.assertRaises(ValueError, svnmerge.RevisionSet, "10;12-15")
         self.assertRaises(ValueError, svnmerge.RevisionSet, "10,foo,3-15")
 
+        self.assertRaises(ValueError, svnmerge.RevisionSet, "10:12:15")
+        self.assertRaises(ValueError, svnmerge.RevisionSet, "10;12:15")
+        self.assertRaises(ValueError, svnmerge.RevisionSet, "10,foo,3:15")
+
     def test_normalized(self):
         rs = svnmerge.RevisionSet("8-15,16-18, 4-6, 9, 18, 1-1, 3-3")
         self.assertEqual(rs.normalized(), [(1,1), (3,6), (8,18)])
         self.assertEqual(str(rs), "1,3-6,8-18")
 
+        rs = svnmerge.RevisionSet("8:15,16:18, 4:6, 9, 18, 1:1, 3:3")
+        self.assertEqual(rs.normalized(), [(1,1), (3,6), (8,18)])
+        self.assertEqual(str(rs), "1,3-6,8-18")
+
+    def test_sorted(self):
+        "Test the sorted() function of the RevisionSet class."
+        rs = svnmerge.RevisionSet("8-15,16-18, 4-6, 9, 18, 1-1, 3-3")
+        self.assertEqual(rs.sorted(), [1, 3, 4, 5, 6, 8, 9, 10, 11,
+                                       12, 13, 14, 15, 16, 17, 18])
+
+        rs = svnmerge.RevisionSet("8:15,16:18, 4:6, 9, 18, 1:1, 3:3")
+        self.assertEqual(rs.sorted(), [1, 3, 4, 5, 6, 8, 9, 10, 11,
+                                       12, 13, 14, 15, 16, 17, 18])
+
     def test_length(self):
         rs = svnmerge.RevisionSet("3-8")
         self.assertEqual(len(rs), 6)
@@ -100,6 +131,13 @@ class TestCase_RevisionSet(unittest.TestCase):
         rs = svnmerge.RevisionSet("1,3,5")
         self.assertEqual(len(rs), 3)
 
+        rs = svnmerge.RevisionSet("3:8")
+        self.assertEqual(len(rs), 6)
+        rs = svnmerge.RevisionSet("3:8,4:10")
+        self.assertEqual(len(rs), 8)
+        rs = svnmerge.RevisionSet("1,3,5")
+        self.assertEqual(len(rs), 3)
+
     def test_iter(self):
         try:
             iter
@@ -109,14 +147,23 @@ class TestCase_RevisionSet(unittest.TestCase):
             rs = svnmerge.RevisionSet("4-13,1-5,34,20-22,18-21")
             self.assertEqual(list(iter(rs)), range(1,14)+range(18,23)+[34])
 
+            rs = svnmerge.RevisionSet("4:13,1:5,34,20:22,18:21")
+            self.assertEqual(list(iter(rs)), range(1,14)+range(18,23)+[34])
+
     def test_union(self):
         rs = svnmerge.RevisionSet("3-8,4-10") | svnmerge.RevisionSet("7-14,1")
         self.assertEqual(str(rs), "1,3-14")
 
+        rs = svnmerge.RevisionSet("3:8,4:10") | svnmerge.RevisionSet("7:14,1")
+        self.assertEqual(str(rs), "1,3-14")
+
     def test_subtraction(self):
         rs = svnmerge.RevisionSet("3-8,4-10") - svnmerge.RevisionSet("7-14,1")
         self.assertEqual(str(rs), "3-6")
 
+        rs = svnmerge.RevisionSet("3:8,4:10") - svnmerge.RevisionSet("7:14,1")
+        self.assertEqual(str(rs), "3-6")
+
     def test_constr_empty(self):
         rs = svnmerge.RevisionSet("")
         self.assertEqual(str(rs), "")
@@ -230,26 +277,26 @@ class TestCase_CommandLineOptions(TestCase_SvnMerge):
         options in any order."""
         self.svnmerge("--log avail",
                       error=True,
-                      match="no integration info")  # accepted
+                      match=r"no integration info")  # accepted
         self.svnmerge("-l avail",
                       error=True,
-                      match="no integration info")  # accepted
+                      match=r"no integration info")  # accepted
         self.svnmerge("-r123 merge",
                       error=True,
-                      match="no integration info")  # accepted
+                      match=r"no integration info")  # accepted
         self.svnmerge("-s -v -r92481 merge",
                       error=True,
-                      match="no integration info")  # accepted
+                      match=r"no integration info")  # accepted
         self.svnmerge("--log merge",
                       error=True,
-                      match="option --log not recognized")
-        self.svnmerge("--diff foobar", error=True, match="foobar")
+                      match=r"option --log not recognized")
+        self.svnmerge("--diff foobar", error=True, match=r"foobar")
 
         # This requires gnu_getopt support to be parsed
         if hasattr(getopt, "gnu_getopt"):
             self.svnmerge("-r123 merge . --log",
                           error=True,
-                          match="option --log not recognized")
+                          match=r"option --log not recognized")
 
 def temp_path():
     try:
@@ -280,6 +327,36 @@ def get_test_path():
 
 class TestCase_TestRepo(TestCase_SvnMerge):
     def setUp(self):
+        """Creates a working copy of a branch at r13 with the
+        following structure, containing revisions (3-6, 13):
+
+          test-branch/
+           test1
+           test2
+           test3
+
+        ...from a repository with the following structure:
+
+          Path                        Created rev
+          ----                        -----------
+          /                           0
+           trunk/                     3
+            test1                     4
+            test2                     5
+            test3                     6
+            test4                     9
+            test5                     10
+           branches/                  1
+            testYYY-branch/           11 (renamed from testXXX-branch in 12)
+             test1                    4
+             test2                    5
+             test3                    6
+            test-branch/              13 (copied from trunk@6)
+             test1                    4
+             test2                    5
+             test3                    6
+           tags/                      2
+        """
         self.cwd = os.getcwd()
 
         test_path = get_test_path()
@@ -353,6 +430,10 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         # completed.
         atexit.register(lambda: rmtree(template_path))
 
+    def tearDown(self):
+        os.chdir(self.cwd)
+        rmtree(self.test_path)
+
     def command_dict(self):
         return dict(TEMPLATE_PATH=self.template_path,
                     TEMPLATE_REPO_PATH=self.template_repo_path,
@@ -380,17 +461,19 @@ class TestCase_TestRepo(TestCase_SvnMerge):
     def testNoWc(self):
         os.mkdir("foo")
         os.chdir("foo")
-        self.svnmerge("init", error=True, match="working dir")
-        self.svnmerge("avail", error=True, match="working dir")
-        self.svnmerge("merge", error=True, match="working dir")
-        self.svnmerge("block", error=True, match="working dir")
-        self.svnmerge("unblock", error=True, match="working dir")
+        self.svnmerge("init", error=True, match=r"working dir")
+        self.svnmerge("avail", error=True, match=r"working dir")
+        self.svnmerge("integrated", error=True, match=r"working dir")
+        self.svnmerge("merge", error=True, match=r"working dir")
+        self.svnmerge("block", error=True, match=r"working dir")
+        self.svnmerge("unblock", error=True, match=r"working dir")
 
     def testCheckNoIntegrationInfo(self):
-        self.svnmerge("avail", error=True, match="no integration")
-        self.svnmerge("merge", error=True, match="no integration")
-        self.svnmerge("block", error=True, match="no integration")
-        self.svnmerge("unblock", error=True, match="no integration")
+        self.svnmerge("avail", error=True, match=r"no integration")
+        self.svnmerge("integrated", error=True, match=r"no integration")
+        self.svnmerge("merge", error=True, match=r"no integration")
+        self.svnmerge("block", error=True, match=r"no integration")
+        self.svnmerge("unblock", error=True, match=r"no integration")
 
     def testBlocked(self):
 
@@ -400,66 +483,81 @@ class TestCase_TestRepo(TestCase_SvnMerge):
                     match=r"Committed revision")
 
         # Block revisions that have already been merged
-        self.svnmerge("block -r5", error=True, match="no available revisions")
+        self.svnmerge("block -r5", error=True, match=r"no available revisions")
 
         # Block phantom revisions
-        self.svnmerge("block -r8", error=True, match="no available revisions")
+        self.svnmerge("block -r8", error=True, match=r"no available revisions")
 
         # Block available revisions
-        self.svnmerge("block -r9", match="'svnmerge-blocked' set")
+        self.svnmerge("block -r9", match=r"'svnmerge-blocked' set")
         self.launch("svn commit -F svnmerge-commit-message.txt",
                     match=r"Committed revision")
 
+        # Check that the revision is still available
+        self.svnmerge("avail", match=r"\A10$")
+
         # Check that the revision was blocked correctly
-        out = self.svnmerge("avail")
-        out = out.rstrip().split("\n")
-        self.assertEqual(out[-1].strip(), "10")
+        self.svnmerge("avail -B", match=r"\A9$")
+
+        # Check that both revisions are available with avail -A
+        self.svnmerge("avail -A", match=r"\A9-10$")
 
         # Block all remaining revisions
-        self.svnmerge("block", match="'svnmerge-blocked' set")
+        self.svnmerge("block", match=r"'svnmerge-blocked' set")
         self.launch("svn commit -F svnmerge-commit-message.txt",
                     match=r"Committed revision")
 
         # Check that all revisions were blocked correctly
-        out = self.svnmerge("avail")
-        out = out.rstrip().split("\n")
-        self.assertEqual(out[-1].strip(), "")
+        self.svnmerge("avail -B", match=r"\A9-10$")
+
+        # Check that all revisions are available using avail -A
+        self.svnmerge("avail -A", match=r"\A9-10$")
+
+        # Check that no revisions are available, now that they have
+        # been blocked
+        self.svnmerge("avail", match=r"\A\Z")
 
         # Unblock all revisions
-        self.svnmerge("unblock", match="'svnmerge-blocked' deleted")
+        self.svnmerge("unblock", match=r"'svnmerge-blocked' deleted")
         self.launch("svn commit -F svnmerge-commit-message.txt",
                     match=r"Committed revision")
 
         # Check that all revisions are available
-        out = self.svnmerge("avail")
-        out = out.rstrip().split("\n")
-        self.assertEqual(out[-1].strip(), "9-10")
+        self.svnmerge("avail", match=r"\A9-10$")
+        self.svnmerge("avail -A", match=r"\A9-10$")
+
+        # Check that no revisions are blocked
+        self.svnmerge("avail -B", match=r"\A$")
 
     def testBasic(self):
         self.svnmerge("init")
         p = self.getproperty()
         self.assertEqual("/trunk:1-6", p)
 
-        out = self.svnmerge("avail -v", match=r"phantom.*7-8")
-        out = out.rstrip().split("\n")
-        self.assertEqual(out[-1].strip(), "9-10")
+        self.svnmerge("avail", match=r"\A9-10$")
+        self.svnmerge("avail -v", match=r"phantom.*7-8")
+
+        self.svnmerge("avail -B", match=r"\A$")
+        self.svnmerge("avail -A", match=r"\A9-10$")
 
         self.svnmerge("avail --log", match=r"| r7.*| r8")
-        self.svnmerge("avail --diff -r9", match="Index: test4")
+        self.svnmerge("avail --diff -r9", match=r"Index: test4")
 
-        out = self.svnmerge("avail --log -r5")
-        self.assertEqual(out.strip(), "")
-        out = self.svnmerge("avail --diff -r5")
-        self.assertEqual(out.strip(), "")
+        self.svnmerge("avail --log -r5", match=r"\A\Z")
+        self.svnmerge("avail --diff -r5", match=r"\A\Z")
 
-    def testCommitFile(self):
-        self.svnmerge("init -vf commit-log.txt", match="wrote commit message")
+        self.svnmerge("integrated", match=r"^3-6$")
+        self.svnmerge("integrated --log -r5", match=r"| r5 ")
+        self.svnmerge("integrated --diff -r5", match=r"Index: test2")
+
+    def test_log_msg_suggest(self):
+        self.svnmerge("init -vf commit-log.txt", match=r"wrote commit message")
         self.assert_(os.path.exists("commit-log.txt"))
         os.remove("commit-log.txt")
 
     def testInitForce(self):
         open("test1", "a").write("foo")
-        self.svnmerge("init", error=True, match="clean")
+        self.svnmerge("init", error=True, match=r"clean")
         self.svnmerge("init -F")
         p = self.getproperty()
         self.assertEqual("/trunk:1-6", p)
@@ -473,11 +571,7 @@ class TestCase_TestRepo(TestCase_SvnMerge):
     def testCheckNoCopyfrom(self):
         os.chdir("..")
         os.chdir("trunk")
-        self.svnmerge("init", error=True, match="no copyfrom")
-
-    def tearDown(self):
-        os.chdir(self.cwd)
-        rmtree(self.test_path)
+        self.svnmerge("init", error=True, match=r"no copyfrom")
 
     def testTrimmedAvailMerge(self):
         """Check that both avail and merge do not search for phantom revs too hard."""
@@ -485,6 +579,17 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         self.svnmerge("avail -vv -r8-9", match=r"svn log.*-r8:9")
         self.svnmerge("merge -F -vv -r8-9", match=r"svn log.*-r8:9")
         self.svnmerge("avail -vv -r2", nonmatch=r"svn log")
+        self.svnmerge("integrated", match=r"^3-6,8-9$")
+
+    def testMergeRecordOnly(self):
+        """Check that flagging revisions as manually merged works."""
+        self.svnmerge("init")
+        self.svnmerge("avail -vv -r9", match=r"svn log.*-r9:9")
+        self.svnmerge("merge --record-only -F -vv -r9",
+                      nonmatch=r"svn merge -r 8:9")
+        self.svnmerge("avail -r9", match=r"\A$")
+        self.svnmerge("integrated", match=r"^3-6,9$")
+        self.svnmerge("integrated -r9", match=r"^9$")
 
     def testBidirectionalMerges(self):
         """Check that reflected revisions are recognized properly for bidirectional merges."""
@@ -512,24 +617,19 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         self.launch("svn commit -m \"Change to test1 on trunk\"",
                     match=r"Committed revision 16")
 
+        self.svnmerge("integrated", match=r"^13-14$")
+
         os.chdir("..")
         os.chdir("test-branch")
 
         # Not using switch, so must update to get latest repository rev.
         self.launch("svn update", match=r"At revision 16")
 
-        self.svnmerge("avail -vv", match=r"15-16$")
-        self.svnmerge("merge -vv", match=r"merge -r 14:16")
+        self.svnmerge("avail -vv --bidirectional", match=r"16$")
+        self.svnmerge("merge -vv --bidirectional", match=r"merge -r 15:16")
         p = self.getproperty()
         self.assertEqual("/trunk:1-16", p)
-
-        # There will be directory property conflict on 'test-branch'
-        # due to the attempted merge from trunk of the addition of the
-        # svnmerge-integrated property, which already exists in the
-        # branch since 'svnmerge.py init' was run in it.  So just
-        # resolve it, as it currently has the correct value (only on
-        # svn 1.3.x, so do not execute a match assertion).
-        self.launch("svn resolved .")
+        self.svnmerge("integrated", match=r"^3-16$")
 
         self.launch("svn commit -F svnmerge-commit-message.txt",
                     match=r"Committed revision 17")
@@ -556,6 +656,8 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         p = self.getproperty()
         self.assertEqual("/branches/test-branch:1-18", p)
 
+        self.svnmerge("integrated", match=r"^13-18$")
+
     def testBidirectionalMergesMultiBranch(self):
         """Check that merges from a second branch are not considered reflected for other branches."""
 
@@ -616,6 +718,9 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         p = self.getproperty()
         self.assertEqual("/branches/test-branch:1-16 /branches/test-branch2:1-19", p)
 
+        self.svnmerge("integrated --head ../test-branch2", match=r"^14-19$")
+        self.svnmerge("integrated --head ../test-branch", match=r"^13-16$")
+
         self.launch("svn commit -F svnmerge-commit-message.txt",
                     match=r"Committed revision 20")
         os.remove("svnmerge-commit-message.txt")
@@ -634,6 +739,8 @@ class TestCase_TestRepo(TestCase_SvnMerge):
         p = self.getproperty()
         self.assertEqual("/trunk:1-20", p)
 
+        self.svnmerge("integrated", match=r"^3-20$")
+
 if __name__ == "__main__":
     # If an existing template repository and working copy for testing
     # exists, then always remove it.  This prevents any problems if
