@@ -69,8 +69,9 @@ public class SVNTests extends TestCase
      */
     protected WC greekWC;
     /**
-     * the directory "local_tmp" in the rootDir. This will be used for the
-     * sample repository and its dumpfile and for the config directory
+     * the directory "svn-test-work/local_tmp" in the rootDir. This
+     * will be used for the sample repository and its dumpfile and for
+     * the config directory
      */
     protected File localTmp;
     /**
@@ -118,21 +119,25 @@ public class SVNTests extends TestCase
      */
     protected static void processArgs(String[] args)
     {
-        if(args == null)
+        if (args == null)
             return;
         for (int i = 0; i < args.length; i++)
         {
             String arg = args[i];
-            if("-d".equals(arg))
-                if(i + 1 < args.length)
+            if ("-d".equals(arg))
+            {
+                if (i + 1 < args.length)
                 {
                     rootDirectoryName = args[++i];
                 }
-            if("-u".equals(arg))
-                if(i + 1 < args.length)
+            }
+            if ("-u".equals(arg))
+            {
+                if (i + 1 < args.length)
                 {
                     rootUrl = args[++i];
                 }
+            }
         }
     }
 
@@ -198,6 +203,9 @@ public class SVNTests extends TestCase
 
         // create a clean directory for the config files and the sample
         // repository
+        //
+        // ### The path is now "svn-test-work/local_tmp", however, I'm
+        // ### not sure how to update this code for that.
         localTmp = new File(rootDir, "local_tmp");
         if(localTmp.exists())
             removeDirectoryWithContent(localTmp);
@@ -207,7 +215,7 @@ public class SVNTests extends TestCase
 
         // create and configure the needed subversion objects
         admin = new SVNAdmin();
-        client = new SVNClient();
+        client = new SVNClientSynchronized();
         client.notification2(new MyNotifier());
         client.commitMessageHandler(new MyCommitMessage());
         client.username("jrandom");
@@ -229,6 +237,10 @@ public class SVNTests extends TestCase
                 new IgnoreOutputer(), null, null, false);
 
         // create the directory for the repositories and the working copies
+        //
+        // ### The paths are now "svn-test-work/repositories" and
+        // ### "svn-test-work/repositories".  However, I'm not sure
+        // ### how to update this code for that. 
         repositories = new File(rootDir, "repositories");
         repositories.mkdirs();
         workingCopies = new File(rootDir, "working_copies");
@@ -352,6 +364,25 @@ public class SVNTests extends TestCase
                 stateFlags, url));
     }
 
+    /**
+     * Intended to be called as part of test method execution
+     * (post-{@link #setUp()}).  Calls <code>fail()</code> if the
+     * directory name cannot be determined.
+     *
+     * @return The name of the working copy administrative directory.
+     * @since 1.3
+     */
+    protected String getAdminDirectoryName() {
+        String admDirName = null;
+        if (this.client != null) {
+            admDirName = client.getAdminDirectoryName();
+        }
+        if (admDirName == null) {
+            fail("Unable to determine the WC admin directory name");
+        }
+        return admDirName;
+    }
+
     /**
      * internal class which implements the OutputInterface to write the data
      * to a file.
@@ -670,7 +701,7 @@ public class SVNTests extends TestCase
          */
         String myPath;
         /**
-         * the kind of node (file, diretory or none, see NodeKind)
+         * the kind of node (file, directory or none, see NodeKind)
          */
         int myNodeKind;
         /**
