@@ -64,7 +64,44 @@ typedef svn_error_t *(svn_opt_subcommand_t)
 #define SVN_OPT_FIRST_LONGOPT_ID 256
 
 
-/** One element of a subcommand dispatch table. */
+/** One element of a subcommand dispatch table.
+ *
+ * @since New in 1.4.
+ */
+typedef struct svn_opt_subcommand_desc2_t
+{
+  /** The full name of this command. */
+  const char *name;
+
+  /** The function this command invokes. */
+  svn_opt_subcommand_t *cmd_func;
+
+  /** A list of alias names for this command (e.g., 'up' for 'update'). */
+  const char *aliases[SVN_OPT_MAX_ALIASES];
+
+  /** A brief string describing this command, for usage messages. */
+  const char *help;
+
+  /** A list of options accepted by this command.  Each value in the
+   * array is a unique enum (the 2nd field in apr_getopt_option_t)
+   */
+  int valid_options[SVN_OPT_MAX_OPTIONS];
+
+  /** A list of option help descriptions, keyed by the option unique enum
+   * (the 2nd field in apr_getopt_option_t), which override the generic
+   * descriptions given in an apr_getopt_option_t on a per-subcommand basis.
+   */
+  struct { int optch; const char *desc; } desc_overrides[SVN_OPT_MAX_OPTIONS];
+} svn_opt_subcommand_desc2_t;
+
+
+/** One element of a subcommand dispatch table.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ *
+ * Like #svn_opt_subcommand_desc2_t but lacking the @c desc_overrides
+ * member.
+ */
 typedef struct svn_opt_subcommand_desc_t
 {
   /** The full name of this command. */
@@ -90,31 +127,83 @@ typedef struct svn_opt_subcommand_desc_t
 /**
  * Return the entry in @a table whose name matches @a cmd_name, or @c NULL if 
  * none.  @a cmd_name may be an alias.
+ *
+ * @since New in 1.4.
+ */  
+const svn_opt_subcommand_desc2_t *
+svn_opt_get_canonical_subcommand2(const svn_opt_subcommand_desc2_t *table,
+                                  const char *cmd_name);
+
+
+/**
+ * Return the entry in @a table whose name matches @a cmd_name, or @c NULL if 
+ * none.  @a cmd_name may be an alias.
+ *
+ * Same as svn_opt_get_canonical_subcommand2(), but acts on
+ * #svn_opt_subcommand_desc_t.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
  */  
 const svn_opt_subcommand_desc_t *
-svn_opt_get_canonical_subcommand (const svn_opt_subcommand_desc_t *table,
-                                  const char *cmd_name);
+svn_opt_get_canonical_subcommand(const svn_opt_subcommand_desc_t *table,
+                                 const char *cmd_name);
+
+
+/**
+ * Return pointer to an @c apr_getopt_option_t for the option whose 
+ * option code is @a code, or @c NULL if no match.  @a option_table must end
+ * with an element whose every field is zero.  If @c command is non-NULL,
+ * then the subcommand-specific option description instead of the generic one,
+ * if a specific description is defined.
+ *
+ * The returned value may be statically allocated, or allocated in @a pool.
+ *
+ * @since New in 1.4.
+ */
+const apr_getopt_option_t *
+svn_opt_get_option_from_code2(int code,
+                              const apr_getopt_option_t *option_table,
+                              const svn_opt_subcommand_desc2_t *command,
+                              apr_pool_t *pool);
 
 
 /**
  * Return the first entry from @a option_table whose option code is @a code,
  * or @c NULL if no match.  @a option_table must end with an element whose
  * every field is zero.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
  */
 const apr_getopt_option_t *
-svn_opt_get_option_from_code (int code,
-                              const apr_getopt_option_t *option_table);
+svn_opt_get_option_from_code(int code,
+                             const apr_getopt_option_t *option_table);
 
 
 /**
  * Return @c TRUE iff subcommand @a command supports option @a option_code,
  * else return @c FALSE.
+ *
+ * @since New in 1.4.
  */
 svn_boolean_t
-svn_opt_subcommand_takes_option (const svn_opt_subcommand_desc_t *command,
+svn_opt_subcommand_takes_option2(const svn_opt_subcommand_desc2_t *command,
                                  int option_code);
 
 
+/**
+ * Return @c TRUE iff subcommand @a command supports option @a option_code,
+ * else return @c FALSE.
+ *
+ * Same as svn_opt_subcommand_takes_option2(), but acts on
+ * #svn_opt_subcommand_desc_t.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ */
+svn_boolean_t
+svn_opt_subcommand_takes_option(const svn_opt_subcommand_desc_t *command,
+                                int option_code);
+
+
 /**
  * Print a generic (not command-specific) usage message to @a stream.
  *
@@ -126,26 +215,42 @@ svn_opt_subcommand_takes_option (const svn_opt_subcommand_desc_t *command,
  * @a footer followed by a newline.
  *
  * Use @a pool for temporary allocation.
+ *
+ * @since New in 1.4.
  */
 void
-svn_opt_print_generic_help (const char *header,
-                            const svn_opt_subcommand_desc_t *cmd_table,
+svn_opt_print_generic_help2(const char *header,
+                            const svn_opt_subcommand_desc2_t *cmd_table,
                             const apr_getopt_option_t *opt_table,
                             const char *footer,
                             apr_pool_t *pool,
                             FILE *stream);
 
 
+/* Same as svn_opt_print_generic_help2(), but acts on
+ * #svn_opt_subcommand_desc_t.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ */
+void
+svn_opt_print_generic_help(const char *header,
+                           const svn_opt_subcommand_desc_t *cmd_table,
+                           const apr_getopt_option_t *opt_table,
+                           const char *footer,
+                           apr_pool_t *pool,
+                           FILE *stream);
+
+
 /**
  * Print an option @a opt nicely into a @a string allocated in @a pool.  
  * If @a doc is set, include the generic documentation string of @a opt,
  * localized to the current locale if a translation is available.
  */
 void
-svn_opt_format_option (const char **string,
-                       const apr_getopt_option_t *opt,
-                       svn_boolean_t doc,
-                       apr_pool_t *pool);
+svn_opt_format_option(const char **string,
+                      const apr_getopt_option_t *opt,
+                      svn_boolean_t doc,
+                      apr_pool_t *pool);
 
 
 
@@ -155,14 +260,29 @@ svn_opt_format_option (const char **string,
  * allocation.  @a subcommand may be a canonical command name or an
  * alias.  (### todo: why does this only print to @c stdout, whereas
  * svn_opt_print_generic_help() gives us a choice?)
+ *
+ * @since New in 1.4.
  */
 void
-svn_opt_subcommand_help (const char *subcommand, 
-                         const svn_opt_subcommand_desc_t *table,
+svn_opt_subcommand_help2(const char *subcommand, 
+                         const svn_opt_subcommand_desc2_t *table,
                          const apr_getopt_option_t *options_table,
                          apr_pool_t *pool);
 
 
+/**
+ * Same as svn_opt_subcommand_help2(), but acts on
+ * #svn_opt_subcommand_desc_t.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ */
+void
+svn_opt_subcommand_help(const char *subcommand, 
+                        const svn_opt_subcommand_desc_t *table,
+                        const apr_getopt_option_t *options_table,
+                        apr_pool_t *pool);
+
+
 
 /* Parsing revision and date options. */
 
@@ -245,10 +365,10 @@ typedef struct svn_opt_revision_t
  *
  * Use @a pool for temporary allocations.
  */
-int svn_opt_parse_revision (svn_opt_revision_t *start_revision,
-                            svn_opt_revision_t *end_revision,
-                            const char *arg,
-                            apr_pool_t *pool);
+int svn_opt_parse_revision(svn_opt_revision_t *start_revision,
+                           svn_opt_revision_t *end_revision,
+                           const char *arg,
+                           apr_pool_t *pool);
 
 
 
@@ -270,10 +390,10 @@ int svn_opt_parse_revision (svn_opt_revision_t *start_revision,
  * @since New in 1.2.
  */
 svn_error_t *
-svn_opt_args_to_target_array2 (apr_array_header_t **targets_p,
-                               apr_getopt_t *os,
-                               apr_array_header_t *known_targets,
-                               apr_pool_t *pool);
+svn_opt_args_to_target_array2(apr_array_header_t **targets_p,
+                              apr_getopt_t *os,
+                              apr_array_header_t *known_targets,
+                              apr_pool_t *pool);
 
 
 /**
@@ -291,13 +411,13 @@ svn_opt_args_to_target_array2 (apr_array_header_t **targets_p,
  * @deprecated Provided for backward compatibility with the 1.1 API.
  */
 svn_error_t *
-svn_opt_args_to_target_array (apr_array_header_t **targets_p,
-                              apr_getopt_t *os,
-                              apr_array_header_t *known_targets,
-                              svn_opt_revision_t *start_revision,
-                              svn_opt_revision_t *end_revision,
-                              svn_boolean_t extract_revisions,
-                              apr_pool_t *pool);
+svn_opt_args_to_target_array(apr_array_header_t **targets_p,
+                             apr_getopt_t *os,
+                             apr_array_header_t *known_targets,
+                             svn_opt_revision_t *start_revision,
+                             svn_opt_revision_t *end_revision,
+                             svn_boolean_t extract_revisions,
+                             apr_pool_t *pool);
 
 
 /**
@@ -307,8 +427,8 @@ svn_opt_args_to_target_array (apr_array_header_t **targets_p,
  * with no arguments. Those commands make use of this function to
  * add "." to the target array if the user passes no args.)
  */
-void svn_opt_push_implicit_dot_target (apr_array_header_t *targets,
-                                       apr_pool_t *pool);
+void svn_opt_push_implicit_dot_target(apr_array_header_t *targets,
+                                      apr_pool_t *pool);
 
 
 /**
@@ -317,10 +437,10 @@ void svn_opt_push_implicit_dot_target (apr_array_header_t *targets,
  * doing any UTF-8 conversion.  Allocate @a *args_p and its values in @a pool.
  */
 svn_error_t *
-svn_opt_parse_num_args (apr_array_header_t **args_p,
-                        apr_getopt_t *os,
-                        int num_args,
-                        apr_pool_t *pool);
+svn_opt_parse_num_args(apr_array_header_t **args_p,
+                       apr_getopt_t *os,
+                       int num_args,
+                       apr_pool_t *pool);
 
 
 /**
@@ -329,9 +449,9 @@ svn_opt_parse_num_args (apr_array_header_t **args_p,
  * Allocate @a *args_p and its values in @a pool.
  */
 svn_error_t *
-svn_opt_parse_all_args (apr_array_header_t **args_p,
-                        apr_getopt_t *os,
-                        apr_pool_t *pool);
+svn_opt_parse_all_args(apr_array_header_t **args_p,
+                       apr_getopt_t *os,
+                       apr_pool_t *pool);
 
 /**
  * Parse a working-copy or URL in @a path, extracting any trailing
@@ -340,20 +460,20 @@ svn_opt_parse_all_args (apr_array_header_t **args_p,
  *
  * Some examples would be:
  *
- *   - foo/bar               -> "foo/bar",       (unspecified)
- *   - foo/bar@@13           -> "foo/bar",       (number, 13)
- *   - foo/bar@@HEAD         -> "foo/bar",       (head)
- *   - foo/bar@@{1999-12-31} -> "foo/bar",       (date, 1999-12-31)
- *   - http://a/b@27         -> "http://a/b",    (number, 27) 
- *   - http://a/b@COMMITTED  -> "http://a/b",    (committed) [*]
- *   - foo/bar@@1:2          -> error
- *   - foo/bar@@baz          -> error
- *   - foo/bar@              -> error
- *   - foo/bar/@@13          -> "foo/bar",       (number, 13)
- *   - foo/bar@@13           -> "foo/bar@",     (number, 13)
- *   - foo/@@bar@@HEAD       -> "foo/@bar",     (head)
- *   - foo@/bar              -> "foo@/bar",     (unspecified)
- *   - foo@@HEAD/bar         -> "foo@HEAD/bar", (unspecified)
+ *   - "foo/bar"              -> "foo/bar",       (unspecified)
+ *   - "foo/bar@13"           -> "foo/bar",       (number, 13)
+ *   - "foo/bar@HEAD"         -> "foo/bar",       (head)
+ *   - "foo/bar@{1999-12-31}" -> "foo/bar",       (date, 1999-12-31)
+ *   - "http://a/b@27"        -> "http://a/b",    (number, 27)
+ *   - "http://a/b@COMMITTED" -> "http://a/b",    (committed) [*]
+ *   - "foo/bar@1:2"          -> error
+ *   - "foo/bar@baz"          -> error
+ *   - "foo/bar@"             -> error
+ *   - "foo/bar/@13"          -> "foo/bar",       (number, 13)
+ *   - "foo/bar@@13"          -> "foo/bar@",      (number, 13)
+ *   - "foo/@bar@HEAD"        -> "foo/@bar",      (head)
+ *   - "foo@/bar"             -> "foo@/bar",      (unspecified)
+ *   - "foo@HEAD/bar"         -> "foo@HEAD/bar",  (unspecified)
  *
  *   [*] Syntactically valid but probably not semantically useful.
  *
@@ -367,10 +487,10 @@ svn_opt_parse_all_args (apr_array_header_t **args_p,
  * @since New in 1.1.
  */
 svn_error_t *
-svn_opt_parse_path (svn_opt_revision_t *rev,
-                    const char **truepath,
-                    const char *path,
-                    apr_pool_t *pool);
+svn_opt_parse_path(svn_opt_revision_t *rev,
+                   const char **truepath,
+                   const char *path,
+                   apr_pool_t *pool);
 
 /**
  * Print either generic help, or command-specific help for @a pgm_name.
@@ -386,7 +506,7 @@ svn_opt_parse_path (svn_opt_revision_t *rev,
  *      information.
  *
  *    - Else if @a print_version is not true, then print generic help,
- *      via svn_opt_print_generic_help() with the @a header, @a cmd_table,
+ *      via svn_opt_print_generic_help2() with the @a header, @a cmd_table,
  *      @a option_table, and @a footer arguments.
  *
  * Use @a pool for temporary allocations.
@@ -395,19 +515,38 @@ svn_opt_parse_path (svn_opt_revision_t *rev,
  * general usage help is that a confused user might put both the
  * --version flag *and* subcommand arguments on a help command line.
  * The logic for handling such a situation should be in one place.
+ *
+ * @since New in 1.4.
  */
 svn_error_t *
-svn_opt_print_help (apr_getopt_t *os,
+svn_opt_print_help2(apr_getopt_t *os,
                     const char *pgm_name,
                     svn_boolean_t print_version,
                     svn_boolean_t quiet,
                     const char *version_footer,
                     const char *header,
-                    const svn_opt_subcommand_desc_t *cmd_table,
+                    const svn_opt_subcommand_desc2_t *cmd_table,
                     const apr_getopt_option_t *option_table,
                     const char *footer,
                     apr_pool_t *pool);
 
+
+/* Same as vn_opt_print_help2), but acts on #svn_opt_subcommand_desc_t.
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ */
+svn_error_t *
+svn_opt_print_help(apr_getopt_t *os,
+                   const char *pgm_name,
+                   svn_boolean_t print_version,
+                   svn_boolean_t quiet,
+                   const char *version_footer,
+                   const char *header,
+                   const svn_opt_subcommand_desc_t *cmd_table,
+                   const apr_getopt_option_t *option_table,
+                   const char *footer,
+                   apr_pool_t *pool);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
