@@ -39,7 +39,7 @@ extern "C" {
 
 /** Subversion error object.
  *
- * Defined here, rather than in svn_error.h, to avoid a recursive #include 
+ * Defined here, rather than in svn_error.h, to avoid a recursive @#include 
  * situation.
  */
 typedef struct svn_error_t
@@ -66,6 +66,12 @@ typedef struct svn_error_t
 
 
 
+/** @defgroup APR_ARRAY_compat_macros APR Array Compatibility Helper Macros
+ * These macros are provided by APR itself from version 1.3.
+ * Definitions are provided here for when using older versions of APR.
+ * @{
+ */
+
 /** index into an apr_array_header_t */
 #ifndef APR_ARRAY_IDX
 #define APR_ARRAY_IDX(ary,i,type) (((type *)(ary)->elts)[i])
@@ -73,9 +79,10 @@ typedef struct svn_error_t
 
 /** easier array-pushing syntax */
 #ifndef APR_ARRAY_PUSH
-#define APR_ARRAY_PUSH(ary,type) (*((type *)apr_array_push (ary)))
+#define APR_ARRAY_PUSH(ary,type) (*((type *)apr_array_push(ary)))
 #endif
 
+/** @} */
 
 /** The various types of nodes in the Subversion filesystem. */
 typedef enum
@@ -96,7 +103,7 @@ typedef enum
 /** About Special Files in Subversion
  *
  * Subversion denotes files that cannot be portably created or
- * modified as special files (svn_node_special).  It stores these
+ * modified as "special" files (svn_node_special).  It stores these
  * files in the repository as a plain text file with the svn:special
  * property set.  The file contents contain: a platform-specific type
  * string, a space character, then any information necessary to create
@@ -113,7 +120,7 @@ typedef enum
  * Special files are stored in the text-base exactly as they are
  * stored in the repository.  The platform specific files are created
  * in the working copy at EOL/keyword translation time using
- * svn_subst_copy_and_translate2.  If the current platform does not
+ * svn_subst_copy_and_translate2().  If the current platform does not
  * support a specific special file type, the file is copied into the
  * working copy as it is seen in the repository.  Because of this,
  * users of other platforms can still view and modify the special
@@ -148,7 +155,7 @@ typedef long int svn_revnum_t;
 /** Convert null-terminated C string @a str to a revision number. */
 #define SVN_STR_TO_REV(str) ((svn_revnum_t) atol(str))
 
-/** In @c printf()-style functions, format revision numbers using this.
+/** In printf()-style functions, format revision numbers using this.
  * Do not use this macro within the Subversion project source code, because
  * the language translation tools have trouble parsing it. */
 #define SVN_REVNUM_T_FMT "ld"
@@ -160,7 +167,7 @@ typedef apr_int64_t svn_filesize_t;
 /** The 'official' invalid file size constant. */
 #define SVN_INVALID_FILESIZE ((svn_filesize_t) -1)
 
-/** In @c printf()-style functions, format file sizes using this. */
+/** In printf()-style functions, format file sizes using this. */
 #define SVN_FILESIZE_T_FMT APR_INT64_T_FMT
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -192,6 +199,38 @@ enum svn_recurse_kind
   svn_recursive
 };
 
+/**
+ * It is sometimes convenient to indicate which parts of an @c svn_dirent_t
+ * object you are actually interested in, so that calculating and sending
+ * the data corresponding to the other fields can be avoided.  These values
+ * can be used for that purpose.
+ *
+ * @defgroup svn_dirent_fields dirent fields
+ * @{
+ */
+
+/** An indication that you are interested in the @c kind field */
+#define SVN_DIRENT_KIND        0x00001
+
+/** An indication that you are interested in the @c size field */
+#define SVN_DIRENT_SIZE        0x00002
+
+/** An indication that you are interested in the @c has_props field */
+#define SVN_DIRENT_HAS_PROPS   0x00004
+
+/** An indication that you are interested in the @c created_rev field */
+#define SVN_DIRENT_CREATED_REV 0x00008
+
+/** An indication that you are interested in the @c time field */
+#define SVN_DIRENT_TIME        0x00010
+
+/** An indication that you are interested in the @c last_author field */
+#define SVN_DIRENT_LAST_AUTHOR 0x00020
+
+/** A combination of all the dirent fields */
+#define SVN_DIRENT_ALL ~((apr_uint32_t ) 0)
+
+/** @} */
 
 /** A general subversion directory entry. */
 typedef struct svn_dirent_t
@@ -214,9 +253,17 @@ typedef struct svn_dirent_t
   /** author of created_rev */
   const char *last_author;
 
+  /* IMPORTANT: If you extend this struct, check svn_dirent_dup(). */
 } svn_dirent_t;
 
 
+/** Return a deep copy of @a dirent, allocated in @a pool.
+ *
+ * @since New in 1.4.
+ */
+svn_dirent_t *svn_dirent_dup(const svn_dirent_t *dirent,
+                             apr_pool_t *pool);
+
 
 
 /** Keyword substitution.
@@ -262,7 +309,8 @@ typedef struct svn_dirent_t
 /** Short version of LastChangedRevision */
 #define SVN_KEYWORD_REVISION_SHORT   "Rev"
 
-/** Medium version of LastChangedRevision, matching the one CVS uses */
+/** Medium version of LastChangedRevision, matching the one CVS uses.
+ * @since New in 1.1. */
 #define SVN_KEYWORD_REVISION_MEDIUM  "Revision"
 
 /** The most recent date (repository time) when this file was changed. */
@@ -288,6 +336,58 @@ typedef struct svn_dirent_t
 
 /** @} */
 
+
+/** All information about a commit.
+ *
+ * @note Objects of this type should always be created using the
+ * svn_create_commit_info() function.
+ *
+ * @since New in 1.3.
+ */
+typedef struct svn_commit_info_t
+{
+  /** just-committed revision. */
+  svn_revnum_t revision;
+
+  /** server-side date of the commit. */
+  const char *date;
+
+  /** author of the commit. */
+  const char *author;
+
+  /** error message from post-commit hook, or NULL. */
+  const char *post_commit_err;
+
+} svn_commit_info_t;
+
+
+/**
+ * Allocate an object of type @c svn_commit_info_t in @a pool and
+ * return it.
+ *
+ * The @c revision field of the new struct is set to @c
+ * SVN_INVALID_REVNUM.  All other fields are initialized to @c NULL.
+ *
+ * @note Any object of the type @c svn_commit_info_t should
+ * be created using this function.
+ * This is to provide for extending the svn_commit_info_t in
+ * the future.
+ *
+ * @since New in 1.3.
+ */
+svn_commit_info_t *
+svn_create_commit_info(apr_pool_t *pool);
+
+
+/**
+ * Return a deep copy @a src_commit_info allocated in @a pool.
+ *
+ * @since New in 1.4.
+ */
+svn_commit_info_t *
+svn_commit_info_dup(const svn_commit_info_t *src_commit_info,
+                    apr_pool_t *pool);
+
 
 /** A structure to represent a path that changed for a log entry. */
 typedef struct svn_log_changed_path_t
@@ -304,8 +404,18 @@ typedef struct svn_log_changed_path_t
 } svn_log_changed_path_t;
 
 
+/**
+ * Return a deep copy of @a changed_path, allocated in @a pool.
+ *
+ * @since New in 1.3.
+ */
+svn_log_changed_path_t *
+svn_log_changed_path_dup(const svn_log_changed_path_t *changed_path,
+                         apr_pool_t *pool);
+
+
 /** The callback invoked by log message loopers, such as
- * @c svn_ra_plugin_t.get_log() and @c svn_repos_get_logs().
+ * @c svn_ra_plugin_t.get_log() and svn_repos_get_logs().
  *
  * This function is invoked once on each log message, in the order
  * determined by the caller (see above-mentioned functions).
@@ -314,18 +424,18 @@ typedef struct svn_log_changed_path_t
  * think they are.  Any of @a author, @a date, or @a message may be @c NULL.
  *
  * If @a date is neither null nor the empty string, it was generated by
- * @c svn_time_to_string() and can be converted to @c apr_time_t with
- * @c svn_time_from_string().
+ * svn_time_to_string() and can be converted to @c apr_time_t with
+ * svn_time_from_string().
  *
  * If @a changed_paths is non-@c NULL, then it contains as keys every path
  * committed in @a revision; the values are (@c svn_log_changed_path_t *) 
  * structures.
  *
  * ### The only reason @a changed_paths is not qualified with `const' is
- * that we usually want to loop over it, and @c apr_hash_first() doesn't
+ * that we usually want to loop over it, and apr_hash_first() doesn't
  * take a const hash, for various reasons.  I'm not sure that those
  * "various reasons" are actually even relevant anymore, and if
- * they're not, it might be nice to change @c apr_hash_first() so
+ * they're not, it might be nice to change apr_hash_first() so
  * read-only uses of hashes can be protected via the type system.
  *
  * Use @a pool for all allocation.  (If the caller is iterating over log
@@ -334,35 +444,78 @@ typedef struct svn_log_changed_path_t
  * clear it after each iteration, destroy it after the loop is done.)
  */
 typedef svn_error_t *(*svn_log_message_receiver_t)
-     (void *baton,
-      apr_hash_t *changed_paths,
-      svn_revnum_t revision,
-      const char *author,
-      const char *date,  /* use svn_time_from_string() if need apr_time_t */
-      const char *message,
-      apr_pool_t *pool);
+  (void *baton,
+   apr_hash_t *changed_paths,
+   svn_revnum_t revision,
+   const char *author,
+   const char *date,  /* use svn_time_from_string() if need apr_time_t */
+   const char *message,
+   apr_pool_t *pool);
 
 
 /** Callback function type for commits.
  *
- * When a commit succeeds, an instance of this is invoked on the @a
- * new_revision, @a date, and @a author of the commit, along with the
- * @a baton closure.
+ * When a commit succeeds, an instance of this is invoked with the
+ * @a commit_info, along with the @a baton closure.
+ * @a pool can be used for temporary allocations.
+ *
+ * @since New in 1.4.
  */
-typedef svn_error_t * (*svn_commit_callback_t) (
-    svn_revnum_t new_revision,
-    const char *date,
-    const char *author,
-    void *baton);
+typedef svn_error_t *(*svn_commit_callback2_t)
+  (const svn_commit_info_t *commit_info,
+   void *baton,
+   apr_pool_t *pool);
+
+/** Same as @c svn_commit_callback2_t, but uses individual
+ * data elements instead of the @c svn_commit_info_t structure
+ *
+ * @deprecated Provided for backward compatibility with the 1.3 API.
+ */
+typedef svn_error_t *(*svn_commit_callback_t)
+  (svn_revnum_t new_revision,
+   const char *date,
+   const char *author,
+   void *baton);
 
 
-/** The maximum amount we (ideally) hold in memory at a time when
+/** Return, in @a *callback2 and @a *callback2_baton a function/baton that
+ * will call @a callback/@a callback_baton, allocating the @a *callback2_baton
+ * in @a pool.
+ *
+ * @note This is used by compatibility wrappers, which exist in more than
+ * Subversion core library.
+ *
+ * @since New in 1.4.
+ */
+void svn_compat_wrap_commit_callback(svn_commit_callback2_t *callback2,
+                                     void **callback2_baton,
+                                     svn_commit_callback_t callback,
+                                     void *callback_baton,
+                                     apr_pool_t *pool);
+
+
+/** A buffer size that may be used when processing a stream of data.
+ *
+ * @note We don't use this constant any longer, since it is considered to be
+ * unnecessarily large.
+ *
+ * @deprecated Provided for backwards compatibility with the 1.3 API.
+ */
+#define SVN_STREAM_CHUNK_SIZE 102400
+
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+/*
+ * The maximum amount we (ideally) hold in memory at a time when
  * processing a stream of data.
  *
  * For example, when copying data from one stream to another, do it in
  * blocks of this size.
+ *
+ * NOTE: This is an internal macro, put here for convenience.
+ * No public API may depend on the particular value of this macro.
  */
-#define SVN_STREAM_CHUNK_SIZE 102400
+#define SVN__STREAM_CHUNK_SIZE 16384
+#endif
 
 /** The maximum amount we can ever hold in memory. */
 /* FIXME: Should this be the same as SVN_STREAM_CHUNK_SIZE? */
@@ -388,8 +541,8 @@ typedef svn_error_t * (*svn_commit_callback_t) (
  * quotes, newlines, or other garbage on the end, such as might be
  * unsafe in an HTTP header.
  */
-svn_error_t *svn_mime_type_validate (const char *mime_type,
-                                     apr_pool_t *pool);
+svn_error_t *svn_mime_type_validate(const char *mime_type,
+                                    apr_pool_t *pool);
 
 
 /** Return false iff @a mime_type is a textual type.
@@ -397,7 +550,7 @@ svn_error_t *svn_mime_type_validate (const char *mime_type,
  * All mime types that start with "text/" are textual, plus some special 
  * cases (for example, "image/x-xbitmap").
  */
-svn_boolean_t svn_mime_type_is_binary (const char *mime_type);
+svn_boolean_t svn_mime_type_is_binary(const char *mime_type);
 
 
 
@@ -406,12 +559,11 @@ svn_boolean_t svn_mime_type_is_binary (const char *mime_type);
  * should continue, the function should return @c SVN_NO_ERROR, if not, it 
  * should return @c SVN_ERR_CANCELLED.
  */
-typedef svn_error_t *(*svn_cancel_func_t) (void *cancel_baton);
+typedef svn_error_t *(*svn_cancel_func_t)(void *cancel_baton);
 
 
 
-/** @since New in 1.2 
- *
+/**
  * A lock object, for client & server to share.
  *
  * A lock represents the exclusive right to add, delete, or modify a
@@ -427,6 +579,8 @@ typedef svn_error_t *(*svn_cancel_func_t) (void *cancel_baton);
  * created with apr_pcalloc(), a default value of 0 is universally safe.
  *
  * @note in the current implementation, only files are lockable.
+ *
+ * @since New in 1.2.
  */
 typedef struct svn_lock_t
 {
@@ -440,23 +594,33 @@ typedef struct svn_lock_t
                                       If value is 0, lock will never expire. */
 } svn_lock_t;
 
-/** @since New in 1.2.
- *
+/**
  * Returns an @c svn_lock_t, allocated in @a pool with all fields initialized
  * to null values.
  *
  * @note To allow for extending the @c svn_lock_t structure in the future
  * releases, this function should always be used to allocate the structure.
- */
-svn_lock_t *
-svn_lock_create (apr_pool_t *pool);
-
-/** @since New in 1.2.
  *
- * Return a deep copy of @a lock, allocated in @a pool.
+ * @since New in 1.2.
  */
 svn_lock_t *
-svn_lock_dup (const svn_lock_t *lock, apr_pool_t *pool);
+svn_lock_create(apr_pool_t *pool);
+
+/**
+ * Return a deep copy of @a lock, allocated in @a pool.
+ *
+ * @since New in 1.2.
+ */
+svn_lock_t *
+svn_lock_dup(const svn_lock_t *lock, apr_pool_t *pool);
+
+/**
+ * Return a formatted Universal Unique IDentifier (UUID) string.
+ *
+ * @since New in 1.4.
+ */
+const char *
+svn_uuid_generate(apr_pool_t *pool);
 
 #ifdef __cplusplus
 }
