@@ -33,16 +33,16 @@
    and (possibly NULL) svn_lock_t LOCK, all allocated in POOL.
    Pointer fields are copied by reference, not dup'd. */
 static svn_error_t *
-build_info_from_dirent (svn_info_t **info,
-                        const svn_dirent_t *dirent,
-                        svn_lock_t *lock,
-                        const char *URL,
-                        svn_revnum_t revision,
-                        const char *repos_UUID,
-                        const char *repos_root,
-                        apr_pool_t *pool)
+build_info_from_dirent(svn_info_t **info,
+                       const svn_dirent_t *dirent,
+                       svn_lock_t *lock,
+                       const char *URL,
+                       svn_revnum_t revision,
+                       const char *repos_UUID,
+                       const char *repos_root,
+                       apr_pool_t *pool)
 {
-  svn_info_t *tmpinfo = apr_pcalloc (pool, sizeof(*tmpinfo));
+  svn_info_t *tmpinfo = apr_pcalloc(pool, sizeof(*tmpinfo));
 
   tmpinfo->URL                  = URL;
   tmpinfo->rev                  = revision;
@@ -62,11 +62,11 @@ build_info_from_dirent (svn_info_t **info,
 /* Helper: build an svn_info_t *INFO struct from svn_wc_entry_t ENTRY,
    allocated in POOL.  Pointer fields are copied by reference, not dup'd. */
 static svn_error_t *
-build_info_from_entry (svn_info_t **info,
-                       const svn_wc_entry_t *entry,
-                       apr_pool_t *pool)
+build_info_from_entry(svn_info_t **info,
+                      const svn_wc_entry_t *entry,
+                      apr_pool_t *pool)
 {
-  svn_info_t *tmpinfo = apr_pcalloc (pool, sizeof(*tmpinfo));
+  svn_info_t *tmpinfo = apr_pcalloc(pool, sizeof(*tmpinfo));
 
   tmpinfo->URL                  = entry->url;
   tmpinfo->rev                  = entry->revision;
@@ -93,7 +93,7 @@ build_info_from_entry (svn_info_t **info,
   /* lock stuff */
   if (entry->lock_token)  /* the token is the critical bit. */
     {
-      tmpinfo->lock = apr_pcalloc (pool, sizeof(*(tmpinfo->lock)));
+      tmpinfo->lock = apr_pcalloc(pool, sizeof(*(tmpinfo->lock)));
 
       tmpinfo->lock->token      = entry->lock_token;
       tmpinfo->lock->owner      = entry->lock_owner;
@@ -106,65 +106,74 @@ build_info_from_entry (svn_info_t **info,
 }
 
 
+/* The dirent fields we care about for our calls to svn_ra_get_dir2. */
+#define DIRENT_FIELDS (SVN_DIRENT_KIND        | \
+                       SVN_DIRENT_CREATED_REV | \
+                       SVN_DIRENT_TIME        | \
+                       SVN_DIRENT_LAST_AUTHOR)
+
+
 /* Helper func for recursively fetching svn_dirent_t's from a remote
    directory and pushing them at an info-receiver callback. */
 static svn_error_t *
-push_dir_info (svn_ra_session_t *ra_session,
-               const char *session_URL,
-               const char *dir,
-               svn_revnum_t rev,
-               const char *repos_UUID,
-               const char *repos_root,
-               svn_info_receiver_t receiver,
-               void *receiver_baton,
-               svn_client_ctx_t *ctx,
-               apr_hash_t *locks,
-               apr_pool_t *pool)
+push_dir_info(svn_ra_session_t *ra_session,
+              const char *session_URL,
+              const char *dir,
+              svn_revnum_t rev,
+              const char *repos_UUID,
+              const char *repos_root,
+              svn_info_receiver_t receiver,
+              void *receiver_baton,
+              svn_client_ctx_t *ctx,
+              apr_hash_t *locks,
+              apr_pool_t *pool)
 {
   apr_hash_t *tmpdirents;
   svn_dirent_t *the_ent;
   svn_info_t *info;
   apr_hash_index_t *hi;
-  apr_pool_t *subpool = svn_pool_create (pool);
+  apr_pool_t *subpool = svn_pool_create(pool);
 
-  SVN_ERR (svn_ra_get_dir (ra_session, dir, rev, &tmpdirents, 
-                           NULL, NULL, pool));
+  SVN_ERR(svn_ra_get_dir2(ra_session, &tmpdirents, NULL, NULL,
+                          dir, rev, DIRENT_FIELDS, pool));
 
-  for (hi = apr_hash_first (pool, tmpdirents); hi; hi = apr_hash_next (hi))
+  for (hi = apr_hash_first(pool, tmpdirents); hi; hi = apr_hash_next(hi))
     {
       const char *path, *URL, *fs_path;
       const void *key;
       svn_lock_t *lock;
       void *val;
 
-      svn_pool_clear (subpool);
+      svn_pool_clear(subpool);
 
       if (ctx->cancel_func)
-        SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
+        SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
 
-      apr_hash_this (hi, &key, NULL, &val);
+      apr_hash_this(hi, &key, NULL, &val);
       the_ent = val;
 
-      path = svn_path_join (dir, key, subpool);
-      URL  = svn_path_url_add_component (session_URL, key, subpool);
+      path = svn_path_join(dir, key, subpool);
+      URL  = svn_path_url_add_component(session_URL, key, subpool);
      
-      fs_path = svn_path_is_child (repos_root, URL, subpool);
-      fs_path = apr_pstrcat (subpool, "/", fs_path, NULL);
-      lock = apr_hash_get (locks, fs_path, APR_HASH_KEY_STRING);
+      fs_path = svn_path_is_child(repos_root, URL, subpool);
+      fs_path = apr_pstrcat(subpool, "/", fs_path, NULL);
+      fs_path = svn_path_uri_decode(fs_path, subpool);
 
-      SVN_ERR (build_info_from_dirent (&info, the_ent, lock, URL, rev,
-                                       repos_UUID, repos_root, subpool));
+      lock = apr_hash_get(locks, fs_path, APR_HASH_KEY_STRING);
 
-      SVN_ERR (receiver (receiver_baton, path, info, subpool));
+      SVN_ERR(build_info_from_dirent(&info, the_ent, lock, URL, rev,
+                                     repos_UUID, repos_root, subpool));
+
+      SVN_ERR(receiver(receiver_baton, path, info, subpool));
 
       if (the_ent->kind == svn_node_dir)
-        SVN_ERR (push_dir_info (ra_session, URL, path,
-                                rev, repos_UUID, repos_root,
-                                receiver, receiver_baton,
-                                ctx, locks, subpool));
+        SVN_ERR(push_dir_info(ra_session, URL, path,
+                              rev, repos_UUID, repos_root,
+                              receiver, receiver_baton,
+                              ctx, locks, subpool));
     }
 
-  svn_pool_destroy (subpool);
+  svn_pool_destroy(subpool);
 
   return SVN_NO_ERROR;
 }
@@ -179,10 +188,10 @@ struct found_entry_baton
 };
 
 static svn_error_t *
-info_found_entry_callback (const char *path,
-                           const svn_wc_entry_t *entry,
-                           void *walk_baton,
-                           apr_pool_t *pool)
+info_found_entry_callback(const char *path,
+                          const svn_wc_entry_t *entry,
+                          void *walk_baton,
+                          apr_pool_t *pool)
 {
   struct found_entry_baton *fe_baton = walk_baton;
   svn_info_t *info;
@@ -191,12 +200,12 @@ info_found_entry_callback (const char *path,
      first one (where it's a child of a parent dir), and only print
      the second one (where we're looking at THIS_DIR.)  */
   if ((entry->kind == svn_node_dir) 
-      && (strcmp (entry->name, SVN_WC_ENTRY_THIS_DIR)))
+      && (strcmp(entry->name, SVN_WC_ENTRY_THIS_DIR)))
     return SVN_NO_ERROR;
 
-  SVN_ERR (build_info_from_entry (&info, entry, pool));
+  SVN_ERR(build_info_from_entry(&info, entry, pool));
  
-  return fe_baton->receiver (fe_baton->receiver_baton, path, info, pool);
+  return fe_baton->receiver(fe_baton->receiver_baton, path, info, pool);
 }
 
 
@@ -211,45 +220,45 @@ entry_walk_callbacks =
 /* Helper function:  push the svn_wc_entry_t for WCPATH at
    RECEIVER/BATON, and possibly recurse over more entries. */
 static svn_error_t *
-crawl_entries (const char *wcpath,
-               svn_info_receiver_t receiver,
-               void *receiver_baton,               
-               svn_boolean_t recurse,
-               svn_client_ctx_t *ctx,
-               apr_pool_t *pool)
+crawl_entries(const char *wcpath,
+              svn_info_receiver_t receiver,
+              void *receiver_baton,               
+              svn_boolean_t recurse,
+              svn_client_ctx_t *ctx,
+              apr_pool_t *pool)
 {
   svn_wc_adm_access_t *adm_access;
   const svn_wc_entry_t *entry;
   svn_info_t *info;
   struct found_entry_baton fe_baton;
   
-  SVN_ERR (svn_wc_adm_probe_open3 (&adm_access, NULL, wcpath, FALSE,
-                                   recurse ? -1 : 0,
-                                   ctx->cancel_func, ctx->cancel_baton,
-                                   pool));
-  SVN_ERR (svn_wc_entry (&entry, wcpath, adm_access, FALSE, pool));
+  SVN_ERR(svn_wc_adm_probe_open3(&adm_access, NULL, wcpath, FALSE,
+                                 recurse ? -1 : 0,
+                                 ctx->cancel_func, ctx->cancel_baton,
+                                 pool));
+  SVN_ERR(svn_wc_entry(&entry, wcpath, adm_access, FALSE, pool));
   if (! entry)
     {
-      return svn_error_createf (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                                _("Cannot read entry for '%s'"), wcpath);
+      return svn_error_createf(SVN_ERR_UNVERSIONED_RESOURCE, NULL,
+                               _("Cannot read entry for '%s'"), wcpath);
     }
 
-  SVN_ERR (build_info_from_entry (&info, entry, pool));
+  SVN_ERR(build_info_from_entry(&info, entry, pool));
   fe_baton.receiver = receiver;
   fe_baton.receiver_baton = receiver_baton;
 
   if (entry->kind == svn_node_file)
-    return receiver (receiver_baton, wcpath, info, pool);
+    return receiver(receiver_baton, wcpath, info, pool);
 
   else if (entry->kind == svn_node_dir)
     {
       if (recurse)
-        SVN_ERR (svn_wc_walk_entries2 (wcpath, adm_access,
-                                       &entry_walk_callbacks, &fe_baton,
-                                       FALSE, ctx->cancel_func,
-                                       ctx->cancel_baton, pool));
+        SVN_ERR(svn_wc_walk_entries2(wcpath, adm_access,
+                                     &entry_walk_callbacks, &fe_baton,
+                                     FALSE, ctx->cancel_func,
+                                     ctx->cancel_baton, pool));
       else
-        return receiver (receiver_baton, wcpath, info, pool);
+        return receiver(receiver_baton, wcpath, info, pool);
     }
       
   return SVN_NO_ERROR;
@@ -257,13 +266,14 @@ crawl_entries (const char *wcpath,
 
 /* Set *SAME_P to TRUE if URL exists in the head of the repository and
    refers to the same resource as it does in REV, using POOL for
-   temporary allocations. */
+   temporary allocations.  RA_SESSION is an open RA session for URL.  */
 static svn_error_t *
-same_resource_in_head (svn_boolean_t *same_p,
-                       const char *url,
-                       svn_revnum_t rev,
-                       svn_client_ctx_t *ctx,
-                       apr_pool_t *pool)
+same_resource_in_head(svn_boolean_t *same_p,
+                      const char *url,
+                      svn_revnum_t rev,
+                      svn_ra_session_t *ra_session,
+                      svn_client_ctx_t *ctx,
+                      apr_pool_t *pool)
 {
   svn_error_t *err;
   svn_opt_revision_t start_rev, end_rev, peg_rev;
@@ -275,16 +285,17 @@ same_resource_in_head (svn_boolean_t *same_p,
   peg_rev.value.number = rev;
   end_rev.kind = svn_opt_revision_unspecified;
 
-  err = svn_client__repos_locations (&head_url, &ignored_rev,
-                                     &ignored_url, &ignored_rev,
-                                     url, &peg_rev,
-                                     &start_rev, &end_rev,
-                                     ctx, pool);
+  err = svn_client__repos_locations(&head_url, &ignored_rev,
+                                    &ignored_url, &ignored_rev,
+                                    ra_session,
+                                    url, &peg_rev,
+                                    &start_rev, &end_rev,
+                                    ctx, pool);
   if (err && 
       ((err->apr_err == SVN_ERR_CLIENT_UNRELATED_RESOURCES) ||
        (err->apr_err == SVN_ERR_FS_NOT_FOUND)))
     {
-      svn_error_clear (err);
+      svn_error_clear(err);
       *same_p = FALSE;
     }
   else if (err)
@@ -293,7 +304,7 @@ same_resource_in_head (svn_boolean_t *same_p,
     {
       /* ### Currently, the URLs should always be equal, since we can't
          ### walk forwards in history. */
-      if (strcmp (url, head_url) == 0)
+      if (strcmp(url, head_url) == 0)
         *same_p = TRUE;
       else
         *same_p = FALSE;
@@ -303,14 +314,14 @@ same_resource_in_head (svn_boolean_t *same_p,
 }
 
 svn_error_t *
-svn_client_info (const char *path_or_url,
-                 const svn_opt_revision_t *peg_revision,
-                 const svn_opt_revision_t *revision,
-                 svn_info_receiver_t receiver,
-                 void *receiver_baton,
-                 svn_boolean_t recurse,
-                 svn_client_ctx_t *ctx,
-                 apr_pool_t *pool)
+svn_client_info(const char *path_or_url,
+                const svn_opt_revision_t *peg_revision,
+                const svn_opt_revision_t *revision,
+                svn_info_receiver_t receiver,
+                void *receiver_baton,
+                svn_boolean_t recurse,
+                svn_client_ctx_t *ctx,
+                apr_pool_t *pool)
 {
   svn_ra_session_t *ra_session, *parent_ra_session;
   svn_revnum_t rev;
@@ -331,9 +342,9 @@ svn_client_info (const char *path_or_url,
           || peg_revision->kind == svn_opt_revision_unspecified))
     {
       /* Do all digging in the working copy. */
-      return crawl_entries (path_or_url,
-                            receiver, receiver_baton,               
-                            recurse, ctx, pool);
+      return crawl_entries(path_or_url,
+                           receiver, receiver_baton,               
+                           recurse, ctx, pool);
     }
 
   /* Go repository digging instead. */
@@ -341,18 +352,18 @@ svn_client_info (const char *path_or_url,
   /* Trace rename history (starting at path_or_url@peg_revision) and
      return RA session to the possibly-renamed URL as it exists in REVISION.
      The ra_session returned will be anchored on this "final" URL. */
-  SVN_ERR (svn_client__ra_session_from_path (&ra_session, &rev,
-                                             &url, path_or_url, peg_revision,
-                                             revision, ctx, pool));
+  SVN_ERR(svn_client__ra_session_from_path(&ra_session, &rev,
+                                           &url, path_or_url, peg_revision,
+                                           revision, ctx, pool));
 
-  SVN_ERR (svn_ra_get_repos_root (ra_session, &repos_root_URL, pool));
-  SVN_ERR (svn_ra_get_uuid (ra_session, &repos_UUID, pool));
+  SVN_ERR(svn_ra_get_repos_root(ra_session, &repos_root_URL, pool));
+  SVN_ERR(svn_ra_get_uuid(ra_session, &repos_UUID, pool));
   
-  svn_path_split (url, &parent_url, &base_name, pool);
+  svn_path_split(url, &parent_url, &base_name, pool);
   base_name = svn_path_uri_decode(base_name, pool);
   
   /* Get the dirent for the URL itself. */
-  err = svn_ra_stat (ra_session, "", rev, &the_ent, pool);
+  err = svn_ra_stat(ra_session, "", rev, &the_ent, pool);
 
   /* svn_ra_stat() will work against old versions of mod_dav_svn, but
      not old versions of svnserve.  In the case of a pre-1.2 svnserve,
@@ -360,15 +371,15 @@ svn_client_info (const char *path_or_url,
   if (err && err->apr_err == SVN_ERR_RA_NOT_IMPLEMENTED)
     {
       /* Fall back to pre-1.2 strategy for fetching dirent's URL. */
-      svn_error_clear (err);
+      svn_error_clear(err);
 
-      SVN_ERR (svn_ra_check_path (ra_session, "", rev, &url_kind, pool));      
+      SVN_ERR(svn_ra_check_path(ra_session, "", rev, &url_kind, pool));      
       if (url_kind == svn_node_none)
-        return svn_error_createf (SVN_ERR_RA_ILLEGAL_URL, NULL,
-                                  _("URL '%s' non-existent in revision '%ld'"),
-                                  url, rev);
+        return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
+                                 _("URL '%s' non-existent in revision '%ld'"),
+                                 url, rev);
 
-      if (strcmp (url, repos_root_URL) == 0)
+      if (strcmp(url, repos_root_URL) == 0)
         {
           /* In this universe, there's simply no way to fetch
              information about the repository's root directory!  So
@@ -379,19 +390,19 @@ svn_client_info (const char *path_or_url,
         }        
       
       /* Open a new RA session to the item's parent. */
-      SVN_ERR (svn_client__open_ra_session_internal (&parent_ra_session,
-                                                     parent_url, NULL,
-                                                     NULL, NULL, FALSE, TRUE, 
-                                                     ctx, pool));
+      SVN_ERR(svn_client__open_ra_session_internal(&parent_ra_session,
+                                                   parent_url, NULL,
+                                                   NULL, NULL, FALSE, TRUE, 
+                                                   ctx, pool));
       
       /* Get all parent's entries, and find the item's dirent in the hash. */
-      SVN_ERR (svn_ra_get_dir (parent_ra_session, "", rev, &parent_ents, 
-                               NULL, NULL, pool));
-      the_ent = apr_hash_get (parent_ents, base_name, APR_HASH_KEY_STRING);
+      SVN_ERR(svn_ra_get_dir2(parent_ra_session, &parent_ents, NULL, NULL,
+                              "", rev, DIRENT_FIELDS, pool));
+      the_ent = apr_hash_get(parent_ents, base_name, APR_HASH_KEY_STRING);
       if (the_ent == NULL)
-        return svn_error_createf (SVN_ERR_RA_ILLEGAL_URL, NULL,
-                                  _("URL '%s' non-existent in revision '%ld'"),
-                                  url, rev);
+        return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
+                                 _("URL '%s' non-existent in revision '%ld'"),
+                                 url, rev);
     }
   else if (err)
     {
@@ -399,23 +410,29 @@ svn_client_info (const char *path_or_url,
     }
   
   if (! the_ent)
-    return svn_error_createf (SVN_ERR_RA_ILLEGAL_URL, NULL,
-                              _("URL '%s' non-existent in revision '%ld'"),
-                              url, rev);
+    return svn_error_createf(SVN_ERR_RA_ILLEGAL_URL, NULL,
+                             _("URL '%s' non-existent in revision '%ld'"),
+                             url, rev);
 
   /* Check if the URL exists in HEAD and refers to the same resource.
-     In this case, we check the repository for a lock on this URL. */
-  SVN_ERR (same_resource_in_head (&related, url, rev, ctx, pool));
+     In this case, we check the repository for a lock on this URL.
+
+     ### There is a possible race here, since HEAD might have changed since
+     ### we checked it.  A solution to this problem could be to do the below
+     ### check in a loop which only terminates if the HEAD revision is the same
+     ### before and after this check.  That could, however, lead to a
+     ### starvation situation instead.  */
+  SVN_ERR(same_resource_in_head(&related, url, rev, ra_session, ctx, pool));
   if (related)
     {
-      err = svn_ra_get_lock (ra_session, &lock, "", pool);
+      err = svn_ra_get_lock(ra_session, &lock, "", pool);
 
       /* An old mod_dav_svn will always work; there's nothing wrong with
          doing a PROPFIND for a property named "DAV:supportedlock". But
          an old svnserve will error. */
       if (err && err->apr_err == SVN_ERR_RA_NOT_IMPLEMENTED)
         {
-          svn_error_clear (err);
+          svn_error_clear(err);
           lock = NULL;
         }
       else if (err)
@@ -425,9 +442,9 @@ svn_client_info (const char *path_or_url,
     lock = NULL;
 
   /* Push the URL's dirent (and lock) at the callback.*/  
-  SVN_ERR (build_info_from_dirent (&info, the_ent, lock, url, rev,
-                                   repos_UUID, repos_root_URL, pool));
-  SVN_ERR (receiver (receiver_baton, base_name, info, pool));
+  SVN_ERR(build_info_from_dirent(&info, the_ent, lock, url, rev,
+                                 repos_UUID, repos_root_URL, pool));
+  SVN_ERR(receiver(receiver_baton, base_name, info, pool));
 
   /* Possibly recurse, using the original RA session. */  
  recurse:
@@ -438,7 +455,7 @@ svn_client_info (const char *path_or_url,
 
       if (peg_revision->kind == svn_opt_revision_head)
         {
-          err = svn_ra_get_locks (ra_session, &locks, "", pool);
+          err = svn_ra_get_locks(ra_session, &locks, "", pool);
           
           /* Catch specific errors thrown by old mod_dav_svn or svnserve. */
           if (err && 
@@ -454,10 +471,10 @@ svn_client_info (const char *path_or_url,
       else
         locks = apr_hash_make(pool); /* use an empty hash */
         
-      SVN_ERR (push_dir_info (ra_session, url, "", rev,
-                              repos_UUID, repos_root_URL,
-                              receiver, receiver_baton,
-                              ctx, locks, pool));
+      SVN_ERR(push_dir_info(ra_session, url, "", rev,
+                            repos_UUID, repos_root_URL,
+                            receiver, receiver_baton,
+                            ctx, locks, pool));
     }
 
   return SVN_NO_ERROR;
@@ -465,37 +482,37 @@ svn_client_info (const char *path_or_url,
 
 
 svn_info_t *
-svn_info_dup (const svn_info_t *info, apr_pool_t *pool)
+svn_info_dup(const svn_info_t *info, apr_pool_t *pool)
 {
-  svn_info_t *dupinfo = apr_palloc (pool, sizeof(*dupinfo));
+  svn_info_t *dupinfo = apr_palloc(pool, sizeof(*dupinfo));
 
   /* Perform a trivial copy ... */
   *dupinfo = *info;
 
   /* ...and then re-copy stuff that needs to be duped into our pool. */
   if (info->URL)
-    dupinfo->URL = apr_pstrdup (pool, info->URL);
+    dupinfo->URL = apr_pstrdup(pool, info->URL);
   if (info->repos_root_URL)
-    dupinfo->repos_root_URL = apr_pstrdup (pool, info->repos_root_URL);
+    dupinfo->repos_root_URL = apr_pstrdup(pool, info->repos_root_URL);
   if (info->repos_UUID)
-    dupinfo->repos_UUID = apr_pstrdup (pool, info->repos_UUID);
+    dupinfo->repos_UUID = apr_pstrdup(pool, info->repos_UUID);
   if (info->last_changed_author)
-    dupinfo->last_changed_author = apr_pstrdup (pool,
-                                                info->last_changed_author);
+    dupinfo->last_changed_author = apr_pstrdup(pool,
+                                               info->last_changed_author);
   if (info->lock)
-    dupinfo->lock = svn_lock_dup (info->lock, pool);
+    dupinfo->lock = svn_lock_dup(info->lock, pool);
   if (info->copyfrom_url)
-    dupinfo->copyfrom_url = apr_pstrdup (pool, info->copyfrom_url);
+    dupinfo->copyfrom_url = apr_pstrdup(pool, info->copyfrom_url);
   if (info->checksum)
-    dupinfo->checksum = apr_pstrdup (pool, info->checksum);
+    dupinfo->checksum = apr_pstrdup(pool, info->checksum);
   if (info->conflict_old)
-    dupinfo->conflict_old = apr_pstrdup (pool, info->conflict_old);
+    dupinfo->conflict_old = apr_pstrdup(pool, info->conflict_old);
   if (info->conflict_new)
-    dupinfo->conflict_new = apr_pstrdup (pool, info->conflict_new);
+    dupinfo->conflict_new = apr_pstrdup(pool, info->conflict_new);
   if (info->conflict_wrk)
-    dupinfo->conflict_wrk = apr_pstrdup (pool, info->conflict_wrk);
+    dupinfo->conflict_wrk = apr_pstrdup(pool, info->conflict_wrk);
   if (info->prejfile)
-    dupinfo->prejfile = apr_pstrdup (pool, info->prejfile);
+    dupinfo->prejfile = apr_pstrdup(pool, info->prejfile);
 
   return dupinfo;
 }
