@@ -28,8 +28,22 @@ extern "C" {
 #endif /* __cplusplus */
 
 
-/*** Repository format number. */
-#define SVN_REPOS__FORMAT_NUMBER     3
+/* Repository format number.
+   
+   Formats 0, 1 and 2 were pre-1.0.
+
+   Format 3 was current for 1.0 through to 1.3.
+
+   Format 4 was an abortive experiment during the development of the
+   locking feature in the lead up to 1.2.
+   
+   Format 5 was new in 1.4, and is the first format which may contain
+   BDB or FSFS filesystems with a FS format other than 1, since prior
+   formats are accepted by some versions of Subversion which do not
+   pay attention to the FS format number.
+*/
+#define SVN_REPOS__FORMAT_NUMBER         5
+#define SVN_REPOS__FORMAT_NUMBER_LEGACY  3
 
 
 /*** Repository layout. ***/
@@ -116,27 +130,27 @@ struct svn_repos_t
 
    USER is the authenticated name of the user starting the commit.  */
 svn_error_t *
-svn_repos__hooks_start_commit (svn_repos_t *repos,
-                               const char *user,
-                               apr_pool_t *pool);
+svn_repos__hooks_start_commit(svn_repos_t *repos,
+                              const char *user,
+                              apr_pool_t *pool);
 
 /* Run the pre-commit hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, return SVN_ERR_REPOS_HOOK_FAILURE.  
 
    TXN_NAME is the name of the transaction that is being committed.  */
 svn_error_t *
-svn_repos__hooks_pre_commit (svn_repos_t *repos,
-                             const char *txn_name,
-                             apr_pool_t *pool);
+svn_repos__hooks_pre_commit(svn_repos_t *repos,
+                            const char *txn_name,
+                            apr_pool_t *pool);
 
 /* Run the post-commit hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, run SVN_ERR_REPOS_HOOK_FAILURE.
 
    REV is the revision that was created as a result of the commit.  */
 svn_error_t *
-svn_repos__hooks_post_commit (svn_repos_t *repos,
-                              svn_revnum_t rev,
-                              apr_pool_t *pool);
+svn_repos__hooks_post_commit(svn_repos_t *repos,
+                             svn_revnum_t rev,
+                             apr_pool_t *pool);
 
 /* Run the pre-revprop-change hook for REPOS.  Use POOL for any
    temporary allocations.  If the hook fails, return
@@ -153,13 +167,13 @@ svn_repos__hooks_post_commit (svn_repos_t *repos,
    written to its stdin.  If the property is being deleted, no data
    will be written. */
 svn_error_t *
-svn_repos__hooks_pre_revprop_change (svn_repos_t *repos,
-                                     svn_revnum_t rev,
-                                     const char *author,
-                                     const char *name,
-                                     const svn_string_t *new_value,
-                                     char action,
-                                     apr_pool_t *pool);
+svn_repos__hooks_pre_revprop_change(svn_repos_t *repos,
+                                    svn_revnum_t rev,
+                                    const char *author,
+                                    const char *name,
+                                    const svn_string_t *new_value,
+                                    char action,
+                                    apr_pool_t *pool);
 
 /* Run the pre-revprop-change hook for REPOS.  Use POOL for any
    temporary allocations.  If the hook fails, return
@@ -175,23 +189,23 @@ svn_repos__hooks_pre_revprop_change (svn_repos_t *repos,
    The old value will be passed to the post-revprop hook on stdin.  If
    the property is being created, no data will be written. */
 svn_error_t *
-svn_repos__hooks_post_revprop_change (svn_repos_t *repos,
-                                      svn_revnum_t rev,
-                                      const char *author,
-                                      const char *name,
-                                      svn_string_t *old_value,
-                                      char action,
-                                      apr_pool_t *pool);
+svn_repos__hooks_post_revprop_change(svn_repos_t *repos,
+                                     svn_revnum_t rev,
+                                     const char *author,
+                                     const char *name,
+                                     svn_string_t *old_value,
+                                     char action,
+                                     apr_pool_t *pool);
 
 /* Run the pre-lock hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, return SVN_ERR_REPOS_HOOK_FAILURE.  
 
    PATH is the path being locked, USERNAME is the person doing it.  */
 svn_error_t *
-svn_repos__hooks_pre_lock (svn_repos_t *repos,
-                           const char *path,
-                           const char *username,
-                           apr_pool_t *pool);
+svn_repos__hooks_pre_lock(svn_repos_t *repos,
+                          const char *path,
+                          const char *username,
+                          apr_pool_t *pool);
 
 /* Run the post-lock hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, return SVN_ERR_REPOS_HOOK_FAILURE.  
@@ -199,20 +213,20 @@ svn_repos__hooks_pre_lock (svn_repos_t *repos,
    PATHS is an array of paths being locked, USERNAME is the person
    who did it.  */
 svn_error_t *
-svn_repos__hooks_post_lock (svn_repos_t *repos,
-                            apr_array_header_t *paths,
-                            const char *username,
-                            apr_pool_t *pool);
+svn_repos__hooks_post_lock(svn_repos_t *repos,
+                           apr_array_header_t *paths,
+                           const char *username,
+                           apr_pool_t *pool);
 
 /* Run the pre-unlock hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, return SVN_ERR_REPOS_HOOK_FAILURE.  
    
    PATH is the path being unlocked, USERNAME is the person doing it.  */
 svn_error_t *
-svn_repos__hooks_pre_unlock (svn_repos_t *repos,
-                             const char *path,
-                             const char *username,
-                             apr_pool_t *pool);
+svn_repos__hooks_pre_unlock(svn_repos_t *repos,
+                            const char *path,
+                            const char *username,
+                            apr_pool_t *pool);
 
 /* Run the post-unlock hook for REPOS.  Use POOL for any temporary
    allocations.  If the hook fails, return SVN_ERR_REPOS_HOOK_FAILURE.  
@@ -220,10 +234,10 @@ svn_repos__hooks_pre_unlock (svn_repos_t *repos,
    PATHS is an array of paths being unlocked, USERNAME is the person
    who did it.  */
 svn_error_t *
-svn_repos__hooks_post_unlock (svn_repos_t *repos,
-                              apr_array_header_t *paths,
-                              const char *username,
-                              apr_pool_t *pool);
+svn_repos__hooks_post_unlock(svn_repos_t *repos,
+                             apr_array_header_t *paths,
+                             const char *username,
+                             apr_pool_t *pool);
 
 
 /*** Utility Functions ***/
@@ -232,12 +246,12 @@ svn_repos__hooks_post_unlock (svn_repos_t *repos,
    different contents, FALSE if they have the same contents.
    Use POOL for temporary allocation. */
 svn_error_t *
-svn_repos__compare_files (svn_boolean_t *changed_p,
-                          svn_fs_root_t *root1,
-                          const char *path1,
-                          svn_fs_root_t *root2,
-                          const char *path2,
-                          apr_pool_t *pool);
+svn_repos__compare_files(svn_boolean_t *changed_p,
+                         svn_fs_root_t *root1,
+                         const char *path1,
+                         svn_fs_root_t *root2,
+                         const char *path2,
+                         apr_pool_t *pool);
 
 
 #ifdef __cplusplus
