@@ -38,6 +38,12 @@
 #define SVN_UTF_NTOU_XLATE_HANDLE "svn-utf-ntou-xlate-handle"
 #define SVN_UTF_UTON_XLATE_HANDLE "svn-utf-uton-xlate-handle"
 
+#ifndef AS400
+#define SVN_APR_UTF8_CHARSET "UTF-8"
+#else
+#define SVN_APR_UTF8_CHARSET (const char*)1208
+#endif
+
 #if APR_HAS_THREADS
 static apr_thread_mutex_t *xlate_handle_mutex = NULL;
 #endif
@@ -73,12 +79,12 @@ static apr_hash_t *xlate_handle_hash = NULL;
 
 /* Clean up the xlate handle cache. */
 static apr_status_t
-xlate_cleanup (void *arg)
+xlate_cleanup(void *arg)
 {
   /* We set the cache variables to NULL so that translation works in other
      cleanup functions, even if it isn't cached then. */
 #if APR_HAS_THREADS
-  apr_thread_mutex_destroy (xlate_handle_mutex);
+  apr_thread_mutex_destroy(xlate_handle_mutex);
   xlate_handle_mutex = NULL;
 #endif
   xlate_handle_hash = NULL;
@@ -88,7 +94,7 @@ xlate_cleanup (void *arg)
 
 /* Set the handle of ARG to NULL. */
 static apr_status_t
-xlate_handle_node_cleanup (void *arg)
+xlate_handle_node_cleanup(void *arg)
 {
   xlate_handle_node_t *node = arg;
 
@@ -97,7 +103,7 @@ xlate_handle_node_cleanup (void *arg)
 }
 
 void
-svn_utf_initialize (apr_pool_t *pool)
+svn_utf_initialize(apr_pool_t *pool)
 {
   apr_pool_t *subpool;
 #if APR_HAS_THREADS
@@ -110,34 +116,65 @@ svn_utf_initialize (apr_pool_t *pool)
          We can't use the pool passed to us by the caller, since we will
          use it for xlate handle allocations, possibly in multiple threads,
          and pool allocation is not thread-safe. */
-      subpool = svn_pool_create (pool);
+      subpool = svn_pool_create(pool);
 #if APR_HAS_THREADS
-      if (apr_thread_mutex_create (&mutex, APR_THREAD_MUTEX_DEFAULT, subpool)
+      if (apr_thread_mutex_create(&mutex, APR_THREAD_MUTEX_DEFAULT, subpool)
           == APR_SUCCESS)
         xlate_handle_mutex = mutex;
       else
         return;
 #endif
       
-      xlate_handle_hash = apr_hash_make (subpool);
-      apr_pool_cleanup_register (subpool, NULL, xlate_cleanup,
-                                 apr_pool_cleanup_null);
+      xlate_handle_hash = apr_hash_make(subpool);
+      apr_pool_cleanup_register(subpool, NULL, xlate_cleanup,
+                                apr_pool_cleanup_null);
     }
 }
 
+/* Return a unique string key based on TOPAGE and FROMPAGE.  TOPAGE and
+ * FROMPAGE can be any valid arguments of the same name to
+ * apr_xlate_open().  Allocate the returned string in POOL. */
+static const char*
+get_xlate_key(const char *topage,
+              const char *frompage,
+              apr_pool_t *pool)
+{
+#ifndef AS400
+  /* In the cases of SVN_APR_LOCALE_CHARSET and SVN_APR_DEFAULT_CHARSET
+   * topage/frompage is really an int, not a valid string.  So generate a
+   * unique key accordingly. */
+  if (frompage == SVN_APR_LOCALE_CHARSET)
+    frompage = "APR_LOCALE_CHARSET";
+  else if (frompage == SVN_APR_DEFAULT_CHARSET)
+    frompage = "APR_DEFAULT_CHARSET";
+
+  if (topage == SVN_APR_LOCALE_CHARSET)
+    topage = "APR_LOCALE_CHARSET";
+  else if (topage == SVN_APR_DEFAULT_CHARSET)
+    topage = "APR_DEFAULT_CHARSET";
+
+  return apr_pstrcat(pool, "svn-utf-", frompage, "to", topage,
+                     "-xlate-handle", NULL);
+#else
+  /* OS400 code pages are always ints. */
+  return apr_psprintf(pool, "svn-utf-%dto%d-xlate-handle", (int)frompage,
+                      (int)topage);
+#endif
+}
+
 /* Set *RET to a handle node for converting from FROMPAGE to TOPAGE,
    creating the handle node if it doesn't exist in USERDATA_KEY.
    If a node is not cached and apr_xlate_open() returns APR_EINVAL or
-   APR_ENOTIMPL, set (*ret)->handle to NULL.  If fail for any other
+   APR_ENOTIMPL, set (*RET)->handle to NULL.  If fail for any other
    reason, return the error.
 
-   Allocate *ret and the xlate handle in POOL if svn_utf_initialize()
+   Allocate *RET and its xlate handle in POOL if svn_utf_initialize()
    hasn't been called or USERDATA_KEY is NULL.  Else, allocate them
    in the pool of xlate_handle_hash. */
 static svn_error_t *
-get_xlate_handle_node (xlate_handle_node_t **ret,
-                       const char *topage, const char *frompage,
-                       const char *userdata_key, apr_pool_t *pool)
+get_xlate_handle_node(xlate_handle_node_t **ret,
+                      const char *topage, const char *frompage,
+                      const char *userdata_key, apr_pool_t *pool)
 {
   xlate_handle_node_t **old_node_p;
   xlate_handle_node_t *old_node = NULL;
@@ -151,13 +188,13 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
       if (xlate_handle_hash)
         {
 #if APR_HAS_THREADS
-          apr_err = apr_thread_mutex_lock (xlate_handle_mutex);
+          apr_err = apr_thread_mutex_lock(xlate_handle_mutex);
           if (apr_err != APR_SUCCESS)
-            return svn_error_create (apr_err, NULL,
-                                     _("Can't lock charset translation mutex"));
+            return svn_error_create(apr_err, NULL,
+                                    _("Can't lock charset translation mutex"));
 #endif
-          old_node_p = apr_hash_get (xlate_handle_hash, userdata_key,
-                                     APR_HASH_KEY_STRING);
+          old_node_p = apr_hash_get(xlate_handle_hash, userdata_key,
+                                    APR_HASH_KEY_STRING);
           if (old_node_p)
             old_node = *old_node_p;
           if (old_node)
@@ -169,11 +206,11 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
                   *old_node_p = old_node->next;
                   old_node->next = NULL;
 #if APR_HAS_THREADS
-                  apr_err = apr_thread_mutex_unlock (xlate_handle_mutex);
+                  apr_err = apr_thread_mutex_unlock(xlate_handle_mutex);
                   if (apr_err != APR_SUCCESS)
-                    return svn_error_create (apr_err, NULL,
-                                             _("Can't unlock charset "
-                                               "translation mutex"));
+                    return svn_error_create(apr_err, NULL,
+                                            _("Can't unlock charset "
+                                              "translation mutex"));
 #endif
                   *ret = old_node;
                   return SVN_NO_ERROR;
@@ -184,7 +221,7 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
         {
           void *p;
           /* We fall back on a per-pool cache instead. */
-          apr_pool_userdata_get (&p, userdata_key, pool);
+          apr_pool_userdata_get(&p, userdata_key, pool);
           old_node = p;
           /* Ensure that the handle is still valid. */
           if (old_node && old_node->valid)
@@ -200,47 +237,66 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
 
   /* The error handling doesn't support the following cases, since we don't
      use them currently.  Catch this here. */
-  assert (frompage != APR_DEFAULT_CHARSET && topage != APR_DEFAULT_CHARSET
-          && (frompage != APR_LOCALE_CHARSET || topage != APR_LOCALE_CHARSET));
+#ifndef AS400
+  /* On OS400 V5R4 with UTF support, APR_DEFAULT_CHARSET and
+   * APR_LOCALE_CHARSET are both UTF-8 (CCSID 1208), so we won't get far
+   * with this assert active. */
+  assert(frompage != SVN_APR_DEFAULT_CHARSET
+         && topage != SVN_APR_DEFAULT_CHARSET
+         && (frompage != SVN_APR_LOCALE_CHARSET
+             || topage != SVN_APR_LOCALE_CHARSET));
+#endif
 
   /* Use the correct pool for creating the handle. */
   if (userdata_key && xlate_handle_hash)
-    pool = apr_hash_pool_get (xlate_handle_hash);
+    pool = apr_hash_pool_get(xlate_handle_hash);
 
   /* Try to create a handle. */
-  apr_err = apr_xlate_open (&handle, topage, frompage, pool);
+#ifndef AS400
+  apr_err = apr_xlate_open(&handle, topage, frompage, pool);
+#else
+  apr_err = apr_xlate_open(&handle, (int)topage, (int)frompage, pool);
+#endif
 
-  if (APR_STATUS_IS_EINVAL (apr_err) || APR_STATUS_IS_ENOTIMPL (apr_err))
+  if (APR_STATUS_IS_EINVAL(apr_err) || APR_STATUS_IS_ENOTIMPL(apr_err))
     handle = NULL;
   else if (apr_err != APR_SUCCESS)
     {
       const char *errstr;
       /* Can't use svn_error_wrap_apr here because it calls functions in
          this file, leading to infinite recursion. */
-      if (frompage == APR_LOCALE_CHARSET)
-        errstr = apr_psprintf (pool,
-                               _("Can't create a character converter from "
-                                 "native encoding to '%s'"), topage);
-      else if (topage == APR_LOCALE_CHARSET)
-        errstr = apr_psprintf (pool,
-                               _("Can't create a character converter from "
-                                 "'%s' to native encoding"), frompage);
+#ifndef AS400
+      if (frompage == SVN_APR_LOCALE_CHARSET)
+        errstr = apr_psprintf(pool,
+                              _("Can't create a character converter from "
+                                "native encoding to '%s'"), topage);
+      else if (topage == SVN_APR_LOCALE_CHARSET)
+        errstr = apr_psprintf(pool,
+                              _("Can't create a character converter from "
+                                "'%s' to native encoding"), frompage);
       else
-        errstr = apr_psprintf (pool,
-                               _("Can't create a character converter from "
-                                 "'%s' to '%s'"), frompage, topage);
-      err = svn_error_create (apr_err, NULL, errstr);
+        errstr = apr_psprintf(pool,
+                              _("Can't create a character converter from "
+                                "'%s' to '%s'"), frompage, topage);
+#else
+      /* Handle the error condition normally prevented by the assert
+       * above. */
+      errstr = apr_psprintf(pool,
+                            _("Can't create a character converter from "
+                              "'%i' to '%i'"), frompage, topage);
+#endif
+      err = svn_error_create(apr_err, NULL, errstr);
       goto cleanup;
     }
 
   /* Allocate and initialize the node. */
-  *ret = apr_palloc (pool, sizeof(xlate_handle_node_t));
+  *ret = apr_palloc(pool, sizeof(xlate_handle_node_t));
   (*ret)->handle = handle;
   (*ret)->valid = TRUE;
-  (*ret)->frompage = ((frompage != APR_LOCALE_CHARSET)
-                      ? apr_pstrdup (pool, frompage) : frompage);
-  (*ret)->topage = ((topage != APR_LOCALE_CHARSET)
-                    ? apr_pstrdup (pool, topage) : topage);
+  (*ret)->frompage = ((frompage != SVN_APR_LOCALE_CHARSET)
+                      ? apr_pstrdup(pool, frompage) : frompage);
+  (*ret)->topage = ((topage != SVN_APR_LOCALE_CHARSET)
+                    ? apr_pstrdup(pool, topage) : topage);
   (*ret)->next = NULL;
 
   /* If we are called from inside a pool cleanup handler, the just created
@@ -249,18 +305,18 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
      To prevent this, we register a cleanup handler that will reset the valid
      flag of our node, so we don't use an invalid handle. */
   if (handle)
-    apr_pool_cleanup_register (pool, *ret, xlate_handle_node_cleanup,
-                               apr_pool_cleanup_null);
+    apr_pool_cleanup_register(pool, *ret, xlate_handle_node_cleanup,
+                              apr_pool_cleanup_null);
 
  cleanup:
   /* Don't need the lock anymore. */
 #if APR_HAS_THREADS
   if (userdata_key && xlate_handle_hash)
     {
-      apr_status_t unlock_err = apr_thread_mutex_unlock (xlate_handle_mutex);
+      apr_status_t unlock_err = apr_thread_mutex_unlock(xlate_handle_mutex);
       if (unlock_err != APR_SUCCESS)
-        return svn_error_create (unlock_err, NULL,
-                                 _("Can't unlock charset translation mutex"));
+        return svn_error_create(unlock_err, NULL,
+                                _("Can't unlock charset translation mutex"));
     }
 #endif
 
@@ -273,50 +329,51 @@ get_xlate_handle_node (xlate_handle_node_t **ret,
    ### Mutex errors here are very weird. Should we handle them "correctly"
    ### even if that complicates error handling in the routines below? */
 static void
-put_xlate_handle_node (xlate_handle_node_t *node,
-                       const char *userdata_key,
-                       apr_pool_t *pool)
+put_xlate_handle_node(xlate_handle_node_t *node,
+                      const char *userdata_key,
+                      apr_pool_t *pool)
 {
-  assert (node->next == NULL);
+  assert(node->next == NULL);
   if (!userdata_key)
     return;
   if (xlate_handle_hash)
     {
       xlate_handle_node_t **node_p;
 #if APR_HAS_THREADS
-      if (apr_thread_mutex_lock (xlate_handle_mutex) != APR_SUCCESS)
-        abort ();
+      if (apr_thread_mutex_lock(xlate_handle_mutex) != APR_SUCCESS)
+        abort();
 #endif
-      node_p = apr_hash_get (xlate_handle_hash, userdata_key,
-                             APR_HASH_KEY_STRING);
+      node_p = apr_hash_get(xlate_handle_hash, userdata_key,
+                            APR_HASH_KEY_STRING);
       if (node_p == NULL)
         {
-          node_p = apr_palloc (apr_hash_pool_get (xlate_handle_hash),
-                               sizeof (*node_p));
+          node_p = apr_palloc(apr_hash_pool_get(xlate_handle_hash),
+                              sizeof(*node_p));
           *node_p = NULL;
-          apr_hash_set (xlate_handle_hash, userdata_key,
-                        APR_HASH_KEY_STRING, node_p);
+          apr_hash_set(xlate_handle_hash, userdata_key,
+                       APR_HASH_KEY_STRING, node_p);
         }
       node->next = *node_p;
       *node_p = node;
 #if APR_HAS_THREADS
-      if (apr_thread_mutex_unlock (xlate_handle_mutex) != APR_SUCCESS)
-        abort ();
+      if (apr_thread_mutex_unlock(xlate_handle_mutex) != APR_SUCCESS)
+        abort();
 #endif
     }
   else
     {
       /* Store it in the per-pool cache. */
-      apr_pool_userdata_set (node, userdata_key, apr_pool_cleanup_null, pool);
+      apr_pool_userdata_set(node, userdata_key, apr_pool_cleanup_null, pool);
     }
 }
 
 /* Return the apr_xlate handle for converting native characters to UTF-8. */
 static svn_error_t *
-get_ntou_xlate_handle_node (xlate_handle_node_t **ret, apr_pool_t *pool)
+get_ntou_xlate_handle_node(xlate_handle_node_t **ret, apr_pool_t *pool)
 {
-  return get_xlate_handle_node (ret, "UTF-8", APR_LOCALE_CHARSET,
-                                SVN_UTF_NTOU_XLATE_HANDLE, pool);
+  return get_xlate_handle_node(ret, SVN_APR_UTF8_CHARSET,
+                               SVN_APR_LOCALE_CHARSET,
+                               SVN_UTF_NTOU_XLATE_HANDLE, pool);
 }
 
 
@@ -326,17 +383,18 @@ get_ntou_xlate_handle_node (xlate_handle_node_t **ret, apr_pool_t *pool)
    set *RET to null and return SVN_NO_ERROR; if fail for some other
    reason, return error. */
 static svn_error_t *
-get_uton_xlate_handle_node (xlate_handle_node_t **ret, apr_pool_t *pool)
+get_uton_xlate_handle_node(xlate_handle_node_t **ret, apr_pool_t *pool)
 {
-  return get_xlate_handle_node (ret, APR_LOCALE_CHARSET, "UTF-8",
-                                SVN_UTF_UTON_XLATE_HANDLE, pool);
+  return get_xlate_handle_node(ret, SVN_APR_LOCALE_CHARSET,
+                               SVN_APR_UTF8_CHARSET,
+                               SVN_UTF_UTON_XLATE_HANDLE, pool);
 }
 
 
 /* Copy LEN bytes of SRC, converting non-ASCII and zero bytes to ?\nnn
    sequences, allocating the result in POOL. */
 static const char *
-fuzzy_escape (const char *src, apr_size_t len, apr_pool_t *pool)
+fuzzy_escape(const char *src, apr_size_t len, apr_pool_t *pool)
 {
   const char *src_orig = src, *src_end = src + len;
   apr_size_t new_len = 0;
@@ -346,7 +404,7 @@ fuzzy_escape (const char *src, apr_size_t len, apr_pool_t *pool)
   /* First count how big a dest string we'll need. */
   while (src < src_end)
     {
-      if (! svn_ctype_isascii (*src) || *src == '\0')
+      if (! svn_ctype_isascii(*src) || *src == '\0')
         new_len += 5;  /* 5 slots, for "?\XXX" */
       else
         new_len += 1;  /* one slot for the 7-bit char */
@@ -355,20 +413,20 @@ fuzzy_escape (const char *src, apr_size_t len, apr_pool_t *pool)
     }
 
   /* Allocate that amount. */
-  new = apr_palloc (pool, new_len + 1);
+  new = apr_palloc(pool, new_len + 1);
 
   new_orig = new;
 
   /* And fill it up. */
   while (src_orig < src_end)
     {
-      if (! svn_ctype_isascii (*src_orig) || src_orig == '\0')
+      if (! svn_ctype_isascii(*src_orig) || src_orig == '\0')
         {
           /* This is the same format as svn_xml_fuzzy_escape uses, but that
              function escapes different characters.  Please keep in sync!
              ### If we add another fuzzy escape somewhere, we should abstract
              ### this out to a common function. */
-          sprintf (new, "?\\%03u", (unsigned char) *src_orig);
+          sprintf(new, "?\\%03u", (unsigned char) *src_orig);
           new += 5;
         }
       else
@@ -388,11 +446,11 @@ fuzzy_escape (const char *src, apr_size_t len, apr_pool_t *pool)
 /* Convert SRC_LENGTH bytes of SRC_DATA in NODE->handle, store the result
    in *DEST, which is allocated in POOL. */
 static svn_error_t *
-convert_to_stringbuf (xlate_handle_node_t *node,
-                      const char *src_data,
-                      apr_size_t src_length,
-                      svn_stringbuf_t **dest,
-                      apr_pool_t *pool)
+convert_to_stringbuf(xlate_handle_node_t *node,
+                     const char *src_data,
+                     apr_size_t src_length,
+                     svn_stringbuf_t **dest,
+                     apr_pool_t *pool)
 {
   apr_size_t buflen = src_length;
   apr_status_t apr_err;
@@ -401,7 +459,7 @@ convert_to_stringbuf (xlate_handle_node_t *node,
   char *destbuf;
 
   /* Initialize *DEST to an empty stringbuf. */
-  *dest = svn_stringbuf_create ("", pool);
+  *dest = svn_stringbuf_create("", pool);
   destbuf = (*dest)->data;
 
   /* Not only does it not make sense to convert an empty string, but
@@ -419,7 +477,7 @@ convert_to_stringbuf (xlate_handle_node_t *node,
 
       /* Ensure that *DEST has sufficient storage for the translated
          result. */
-      svn_stringbuf_ensure (*dest, buflen + 1);
+      svn_stringbuf_ensure(*dest, buflen + 1);
 
       /* Update the destination buffer pointer to the first character
          after already-converted output. */
@@ -429,11 +487,11 @@ convert_to_stringbuf (xlate_handle_node_t *node,
       destlen = buflen - (*dest)->len;
 
       /* Attempt the conversion. */
-      apr_err = apr_xlate_conv_buffer (node->handle,
-                                       src_data + (src_length - srclen), 
-                                       &srclen,
-                                       destbuf, 
-                                       &destlen);
+      apr_err = apr_xlate_conv_buffer(node->handle,
+                                      src_data + (src_length - srclen), 
+                                      &srclen,
+                                      destbuf, 
+                                      &destlen);
 
       /* Now, update the *DEST->len to track the amount of output data
          churned out so far from this loop. */
@@ -449,11 +507,12 @@ convert_to_stringbuf (xlate_handle_node_t *node,
 
       /* Can't use svn_error_wrap_apr here because it calls functions in
          this file, leading to infinite recursion. */
-      if (node->frompage == APR_LOCALE_CHARSET)
+#ifndef AS400
+      if (node->frompage == SVN_APR_LOCALE_CHARSET)
         errstr = apr_psprintf
           (pool, _("Can't convert string from native encoding to '%s':"),
            node->topage);
-      else if (node->topage == APR_LOCALE_CHARSET)
+      else if (node->topage == SVN_APR_LOCALE_CHARSET)
         errstr = apr_psprintf
           (pool, _("Can't convert string from '%s' to native encoding:"),
            node->frompage);
@@ -461,9 +520,16 @@ convert_to_stringbuf (xlate_handle_node_t *node,
         errstr = apr_psprintf
           (pool, _("Can't convert string from '%s' to '%s':"),
            node->frompage, node->topage);
-      err = svn_error_create (apr_err, NULL, fuzzy_escape (src_data,
-                                                           src_length, pool));
-      return svn_error_create (apr_err, err, errstr);
+#else
+      /* On OS400 V5R4 every possible node->topage and node->frompage
+       * *really* is an int. */
+      errstr = apr_psprintf
+        (pool, _("Can't convert string from CCSID '%i' to CCSID '%i'"),
+         node->frompage, node->topage);
+#endif
+      err = svn_error_create(apr_err, NULL, fuzzy_escape(src_data,
+                                                         src_length, pool));
+      return svn_error_create(apr_err, err, errstr);
     }
   /* Else, exited due to success.  Trim the result buffer down to the
      right length. */
@@ -478,15 +544,15 @@ convert_to_stringbuf (xlate_handle_node_t *node,
    characters, finding the error pool from POOL.  Otherwise, return
    SVN_NO_ERROR. */
 static svn_error_t *
-check_non_ascii (const char *data, apr_size_t len, apr_pool_t *pool)
+check_non_ascii(const char *data, apr_size_t len, apr_pool_t *pool)
 {
   const char *data_start = data;
 
   for (; len > 0; --len, data++)
     {
-      if ((! apr_isascii (*data))
-          || ((! apr_isspace (*data))
-              && apr_iscntrl (*data)))
+      if ((! apr_isascii(*data))
+          || ((! apr_isspace(*data))
+              && apr_iscntrl(*data)))
         {
           /* Show the printable part of the data, followed by the
              decimal code of the questionable character.  Because if a
@@ -499,7 +565,7 @@ check_non_ascii (const char *data, apr_size_t len, apr_pool_t *pool)
           if (data - data_start)
             {
               const char *error_data
-                = apr_pstrndup (pool, data_start, (data - data_start));
+                = apr_pstrndup(pool, data_start, (data - data_start));
 
               return svn_error_createf
                 (APR_EINVAL, NULL,
@@ -527,9 +593,9 @@ check_non_ascii (const char *data, apr_size_t len, apr_pool_t *pool)
  * is encoded.
  */
 static svn_error_t *
-invalid_utf8 (const char *data, apr_size_t len, apr_pool_t *pool)
+invalid_utf8(const char *data, apr_size_t len, apr_pool_t *pool)
 {
-  const char *last = svn_utf__last_valid (data, len);
+  const char *last = svn_utf__last_valid(data, len);
   const char *valid_txt = "", *invalid_txt = "";
   int i, valid, invalid;
 
@@ -539,101 +605,101 @@ invalid_utf8 (const char *data, apr_size_t len, apr_pool_t *pool)
   if (valid > 24)
     valid = 24;
   for (i = 0; i < valid; ++i)
-    valid_txt = apr_pstrcat (pool, valid_txt,
-                             apr_psprintf (pool, " %02x",
-                                           (unsigned char)last[i-valid]), NULL);
+    valid_txt = apr_pstrcat(pool, valid_txt,
+                            apr_psprintf(pool, " %02x",
+                                         (unsigned char)last[i-valid]), NULL);
 
   /* 4 invalid octets will guarantee that the faulty octet is displayed */
   invalid = data + len - last;
   if (invalid > 4)
     invalid = 4;
   for (i = 0; i < invalid; ++i)
-    invalid_txt = apr_pstrcat (pool, invalid_txt,
-                               apr_psprintf (pool, " %02x",
-                                             (unsigned char)last[i]), NULL);
+    invalid_txt = apr_pstrcat(pool, invalid_txt,
+                              apr_psprintf(pool, " %02x",
+                                           (unsigned char)last[i]), NULL);
 
-  return svn_error_createf (APR_EINVAL, NULL,
-                            _("Valid UTF-8 data\n(hex:%s)\n"
-                              "followed by invalid UTF-8 sequence\n(hex:%s)"),
-                            valid_txt, invalid_txt);
+  return svn_error_createf(APR_EINVAL, NULL,
+                           _("Valid UTF-8 data\n(hex:%s)\n"
+                             "followed by invalid UTF-8 sequence\n(hex:%s)"),
+                           valid_txt, invalid_txt);
 }
 
 /* Verify that the sequence DATA of length LEN is valid UTF-8 */
 static svn_error_t *
-check_utf8 (const char *data, apr_size_t len, apr_pool_t *pool)
+check_utf8(const char *data, apr_size_t len, apr_pool_t *pool)
 {
-  if (! svn_utf__is_valid (data, len))
-    return invalid_utf8 (data, len, pool);
+  if (! svn_utf__is_valid(data, len))
+    return invalid_utf8(data, len, pool);
   return SVN_NO_ERROR;
 }
 
 /* Verify that the NULL terminated sequence DATA is valid UTF-8 */
 static svn_error_t *
-check_cstring_utf8 (const char *data, apr_pool_t *pool)
+check_cstring_utf8(const char *data, apr_pool_t *pool)
 {
 
-  if (! svn_utf__cstring_is_valid (data))
-    return invalid_utf8 (data, strlen (data), pool);
+  if (! svn_utf__cstring_is_valid(data))
+    return invalid_utf8(data, strlen(data), pool);
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
-svn_utf_stringbuf_to_utf8 (svn_stringbuf_t **dest,
-                           const svn_stringbuf_t *src,
-                           apr_pool_t *pool)
+svn_utf_stringbuf_to_utf8(svn_stringbuf_t **dest,
+                          const svn_stringbuf_t *src,
+                          apr_pool_t *pool)
 {
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_ntou_xlate_handle_node (&node, pool));
+  SVN_ERR(get_ntou_xlate_handle_node(&node, pool));
 
   if (node->handle)
     {
-      err = convert_to_stringbuf (node, src->data, src->len, dest, pool);
+      err = convert_to_stringbuf(node, src->data, src->len, dest, pool);
       if (! err)
-        err = check_utf8 ((*dest)->data, (*dest)->len, pool);
+        err = check_utf8((*dest)->data, (*dest)->len, pool);
     }
   else
     {
-      err = check_non_ascii (src->data, src->len, pool);
+      err = check_non_ascii(src->data, src->len, pool);
       if (! err)
-        *dest = svn_stringbuf_dup (src, pool);
+        *dest = svn_stringbuf_dup(src, pool);
     }
 
-  put_xlate_handle_node (node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
+  put_xlate_handle_node(node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
 
   return err;
 }
 
 
 svn_error_t *
-svn_utf_string_to_utf8 (const svn_string_t **dest,
-                        const svn_string_t *src,
-                        apr_pool_t *pool)
+svn_utf_string_to_utf8(const svn_string_t **dest,
+                       const svn_string_t *src,
+                       apr_pool_t *pool)
 {
   svn_stringbuf_t *destbuf;
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_ntou_xlate_handle_node (&node, pool));
+  SVN_ERR(get_ntou_xlate_handle_node(&node, pool));
 
   if (node->handle)
     {
-      err = convert_to_stringbuf (node, src->data, src->len, &destbuf, pool);
+      err = convert_to_stringbuf(node, src->data, src->len, &destbuf, pool);
       if (! err)
-        err = check_utf8 (destbuf->data, destbuf->len, pool);
+        err = check_utf8(destbuf->data, destbuf->len, pool);
       if (! err)
-        *dest = svn_string_create_from_buf (destbuf, pool);
+        *dest = svn_string_create_from_buf(destbuf, pool);
     }
   else
     {
-      err = check_non_ascii (src->data, src->len, pool);
+      err = check_non_ascii(src->data, src->len, pool);
       if (! err)
-        *dest = svn_string_dup (src, pool);
+        *dest = svn_string_dup(src, pool);
     }
 
-  put_xlate_handle_node (node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
+  put_xlate_handle_node(node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
 
   return err;
 }
@@ -644,184 +710,210 @@ svn_utf_string_to_utf8 (const svn_string_t **dest,
    svn_utf_cstring_from_utf8_ex. Convert SRC to DEST using NODE->handle as
    the translator and allocating from POOL. */
 static svn_error_t *
-convert_cstring (const char **dest,
-                 const char *src,
-                 xlate_handle_node_t *node,
-                 apr_pool_t *pool)
+convert_cstring(const char **dest,
+                const char *src,
+                xlate_handle_node_t *node,
+                apr_pool_t *pool)
 {
   if (node->handle)
     {
       svn_stringbuf_t *destbuf;
-      SVN_ERR (convert_to_stringbuf (node, src, strlen (src),
-                                     &destbuf, pool));
+      SVN_ERR(convert_to_stringbuf(node, src, strlen(src),
+                                   &destbuf, pool));
       *dest = destbuf->data;
     }
   else
     {
-      apr_size_t len = strlen (src);
-      SVN_ERR (check_non_ascii (src, len, pool));
-      *dest = apr_pstrmemdup (pool, src, len);
+      apr_size_t len = strlen(src);
+      SVN_ERR(check_non_ascii(src, len, pool));
+      *dest = apr_pstrmemdup(pool, src, len);
     }
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
-svn_utf_cstring_to_utf8 (const char **dest,
-                         const char *src,
-                         apr_pool_t *pool)
+svn_utf_cstring_to_utf8(const char **dest,
+                        const char *src,
+                        apr_pool_t *pool)
 {
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_ntou_xlate_handle_node (&node, pool));
-  err = convert_cstring (dest, src, node, pool);
-  put_xlate_handle_node (node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
-  SVN_ERR (err);
-  SVN_ERR (check_cstring_utf8 (*dest, pool));
+  SVN_ERR(get_ntou_xlate_handle_node(&node, pool));
+  err = convert_cstring(dest, src, node, pool);
+  put_xlate_handle_node(node, SVN_UTF_NTOU_XLATE_HANDLE, pool);
+  SVN_ERR(err);
+  SVN_ERR(check_cstring_utf8(*dest, pool));
 
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
-svn_utf_cstring_to_utf8_ex (const char **dest,
+svn_utf_cstring_to_utf8_ex2(const char **dest,
                             const char *src,
                             const char *frompage,
-                            const char *convset_key,
+                            apr_pool_t *pool)
+{
+  xlate_handle_node_t *node;
+  svn_error_t *err;
+  const char *convset_key = get_xlate_key(SVN_APR_UTF8_CHARSET, frompage,
+                                          pool);
+
+  SVN_ERR(get_xlate_handle_node(&node, SVN_APR_UTF8_CHARSET, frompage,
+                                convset_key, pool));
+  err = convert_cstring(dest, src, node, pool);
+  put_xlate_handle_node(node, convset_key, pool);
+  SVN_ERR(err);
+  SVN_ERR(check_cstring_utf8(*dest, pool));
+
+  return SVN_NO_ERROR;
+}
+
+
+svn_error_t *
+svn_utf_cstring_to_utf8_ex(const char **dest,
+                           const char *src,
+                           const char *frompage,
+                           const char *convset_key,
+                           apr_pool_t *pool)
+{
+  return svn_utf_cstring_to_utf8_ex2(dest, src, frompage, pool);
+}
+
+
+svn_error_t *
+svn_utf_stringbuf_from_utf8(svn_stringbuf_t **dest,
+                            const svn_stringbuf_t *src,
                             apr_pool_t *pool)
 {
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_xlate_handle_node (&node, "UTF-8", frompage, convset_key, pool));
-  err = convert_cstring (dest, src, node, pool);
-  put_xlate_handle_node (node, convset_key, pool);
-  SVN_ERR (err);
-  SVN_ERR (check_cstring_utf8 (*dest, pool));
-
-  return SVN_NO_ERROR;
-}
-
-
-svn_error_t *
-svn_utf_stringbuf_from_utf8 (svn_stringbuf_t **dest,
-                             const svn_stringbuf_t *src,
-                             apr_pool_t *pool)
-{
-  xlate_handle_node_t *node;
-  svn_error_t *err;
-
-  SVN_ERR (get_uton_xlate_handle_node (&node, pool));
+  SVN_ERR(get_uton_xlate_handle_node(&node, pool));
 
   if (node->handle)
     {
-      err = check_utf8 (src->data, src->len, pool);
+      err = check_utf8(src->data, src->len, pool);
       if (! err)
-        err = convert_to_stringbuf (node, src->data, src->len, dest, pool);
+        err = convert_to_stringbuf(node, src->data, src->len, dest, pool);
     }
   else
     {
-      err = check_non_ascii (src->data, src->len, pool);
+      err = check_non_ascii(src->data, src->len, pool);
       if (! err)
-        *dest = svn_stringbuf_dup (src, pool);
+        *dest = svn_stringbuf_dup(src, pool);
     }
 
-  put_xlate_handle_node (node, SVN_UTF_UTON_XLATE_HANDLE, pool);
+  put_xlate_handle_node(node, SVN_UTF_UTON_XLATE_HANDLE, pool);
 
   return err;
 }
 
 
 svn_error_t *
-svn_utf_string_from_utf8 (const svn_string_t **dest,
-                          const svn_string_t *src,
-                          apr_pool_t *pool)
+svn_utf_string_from_utf8(const svn_string_t **dest,
+                         const svn_string_t *src,
+                         apr_pool_t *pool)
 {
   svn_stringbuf_t *dbuf;
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_uton_xlate_handle_node (&node, pool));
+  SVN_ERR(get_uton_xlate_handle_node(&node, pool));
 
   if (node->handle)
     {
-      err = check_utf8 (src->data, src->len, pool);
+      err = check_utf8(src->data, src->len, pool);
       if (! err)
-        err = convert_to_stringbuf (node, src->data, src->len,
-                                    &dbuf, pool);
+        err = convert_to_stringbuf(node, src->data, src->len,
+                                   &dbuf, pool);
       if (! err)
-        *dest = svn_string_create_from_buf (dbuf, pool);
+        *dest = svn_string_create_from_buf(dbuf, pool);
     }
   else
     {
-      err = check_non_ascii (src->data, src->len, pool);
+      err = check_non_ascii(src->data, src->len, pool);
       if (! err)
-        *dest = svn_string_dup (src, pool);
+        *dest = svn_string_dup(src, pool);
     }
 
-  put_xlate_handle_node (node, SVN_UTF_UTON_XLATE_HANDLE, pool);
+  put_xlate_handle_node(node, SVN_UTF_UTON_XLATE_HANDLE, pool);
 
   return err;
 }
 
 
 svn_error_t *
-svn_utf_cstring_from_utf8 (const char **dest,
-                           const char *src,
-                           apr_pool_t *pool)
+svn_utf_cstring_from_utf8(const char **dest,
+                          const char *src,
+                          apr_pool_t *pool)
 {
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (check_utf8 (src, strlen (src), pool));
+  SVN_ERR(check_utf8(src, strlen(src), pool));
 
-  SVN_ERR (get_uton_xlate_handle_node (&node, pool));
-  err = convert_cstring (dest, src, node, pool);
-  put_xlate_handle_node (node, SVN_UTF_UTON_XLATE_HANDLE, pool);
+  SVN_ERR(get_uton_xlate_handle_node(&node, pool));
+  err = convert_cstring(dest, src, node, pool);
+  put_xlate_handle_node(node, SVN_UTF_UTON_XLATE_HANDLE, pool);
 
   return err;
 }
 
 
 svn_error_t *
-svn_utf_cstring_from_utf8_ex (const char **dest,
+svn_utf_cstring_from_utf8_ex2(const char **dest,
                               const char *src,
                               const char *topage,
-                              const char *convset_key,
                               apr_pool_t *pool)
 {
   xlate_handle_node_t *node;
   svn_error_t *err;
+  const char *convset_key = get_xlate_key(topage, SVN_APR_UTF8_CHARSET,
+                                          pool);
 
-  SVN_ERR (check_utf8 (src, strlen (src), pool));
+  SVN_ERR(check_utf8(src, strlen(src), pool));
 
-  SVN_ERR (get_xlate_handle_node (&node, topage, "UTF-8", convset_key, pool));
-  err = convert_cstring (dest, src, node, pool);
-  put_xlate_handle_node (node, convset_key, pool);
+  SVN_ERR(get_xlate_handle_node(&node, topage, SVN_APR_UTF8_CHARSET,
+                                convset_key, pool));
+  err = convert_cstring(dest, src, node, pool);
+  put_xlate_handle_node(node, convset_key, pool);
 
   return err;
 }
 
 
+svn_error_t *
+svn_utf_cstring_from_utf8_ex(const char **dest,
+                             const char *src,
+                             const char *topage,
+                             const char *convset_key,
+                             apr_pool_t *pool)
+{
+  return svn_utf_cstring_from_utf8_ex2(dest, src, topage, pool);
+}
+
+
 const char *
-svn_utf__cstring_from_utf8_fuzzy (const char *src,
-                                  apr_pool_t *pool,
-                                  svn_error_t *(*convert_from_utf8)
-                                  (const char **, const char *, apr_pool_t *))
+svn_utf__cstring_from_utf8_fuzzy(const char *src,
+                                 apr_pool_t *pool,
+                                 svn_error_t *(*convert_from_utf8)
+                                 (const char **, const char *, apr_pool_t *))
 {
   const char *escaped, *converted;
   svn_error_t *err;
 
-  escaped = fuzzy_escape (src, strlen (src), pool);
+  escaped = fuzzy_escape(src, strlen(src), pool);
 
   /* Okay, now we have a *new* UTF-8 string, one that's guaranteed to
      contain only 7-bit bytes :-).  Recode to native... */
-  err = convert_from_utf8 (((const char **) &converted), escaped, pool);
+  err = convert_from_utf8(((const char **) &converted), escaped, pool);
 
   if (err)
     {
-      svn_error_clear (err);
+      svn_error_clear(err);
       return escaped;
     }
   else
@@ -835,22 +927,22 @@ svn_utf__cstring_from_utf8_fuzzy (const char *src,
 
 
 const char *
-svn_utf_cstring_from_utf8_fuzzy (const char *src,
-                                 apr_pool_t *pool)
+svn_utf_cstring_from_utf8_fuzzy(const char *src,
+                                apr_pool_t *pool)
 {
-  return svn_utf__cstring_from_utf8_fuzzy (src, pool,
-                                           svn_utf_cstring_from_utf8);
+  return svn_utf__cstring_from_utf8_fuzzy(src, pool,
+                                          svn_utf_cstring_from_utf8);
 }
 
 
 svn_error_t *
-svn_utf_cstring_from_utf8_stringbuf (const char **dest,
-                                     const svn_stringbuf_t *src,
-                                     apr_pool_t *pool)
+svn_utf_cstring_from_utf8_stringbuf(const char **dest,
+                                    const svn_stringbuf_t *src,
+                                    apr_pool_t *pool)
 {
   svn_stringbuf_t *destbuf;
 
-  SVN_ERR (svn_utf_stringbuf_from_utf8 (&destbuf, src, pool));
+  SVN_ERR(svn_utf_stringbuf_from_utf8(&destbuf, src, pool));
   *dest = destbuf->data;
 
   return SVN_NO_ERROR;
@@ -858,33 +950,33 @@ svn_utf_cstring_from_utf8_stringbuf (const char **dest,
 
 
 svn_error_t *
-svn_utf_cstring_from_utf8_string (const char **dest,
-                                  const svn_string_t *src,
-                                  apr_pool_t *pool)
+svn_utf_cstring_from_utf8_string(const char **dest,
+                                 const svn_string_t *src,
+                                 apr_pool_t *pool)
 {
   svn_stringbuf_t *dbuf;
   xlate_handle_node_t *node;
   svn_error_t *err;
 
-  SVN_ERR (get_uton_xlate_handle_node (&node, pool));
+  SVN_ERR(get_uton_xlate_handle_node(&node, pool));
 
   if (node->handle)
     {
-      err = check_utf8 (src->data, src->len, pool);
+      err = check_utf8(src->data, src->len, pool);
       if (! err)
-        err = convert_to_stringbuf (node, src->data, src->len,
-                                    &dbuf, pool);
+        err = convert_to_stringbuf(node, src->data, src->len,
+                                   &dbuf, pool);
       if (! err)
         *dest = dbuf->data;
     }
   else
     {
-      err = check_non_ascii (src->data, src->len, pool);
+      err = check_non_ascii(src->data, src->len, pool);
       if (! err)
-        *dest = apr_pstrmemdup (pool, src->data, src->len);
+        *dest = apr_pstrmemdup(pool, src->data, src->len);
     }
 
-  put_xlate_handle_node (node, SVN_UTF_UTON_XLATE_HANDLE, pool);
+  put_xlate_handle_node(node, SVN_UTF_UTON_XLATE_HANDLE, pool);
 
   return err;
 }
